// -*- C++ -*-
/*!
 * @file  CorbaPort.h
 * @brief CorbaPort class
 * @date  $Date: 2007-12-31 03:08:02 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: CorbaPort.h 2259 2012-01-04 13:18:29Z n-ando $
 *
 */

#ifndef RTC_CORBAPORT_H
#define RTC_CORBAPORT_H

#include <rtm/PortBase.h>
#include <rtm/CorbaConsumer.h>
#include <rtm/NVUtil.h>
#include <list>

/*!
 * @if jp
 * @namespace RTC
 *
 * @brief RTR|[lg
 *
 * @else
 *
 * @namespace RTC
 *
 * @brief RT-Component
 *
 * @endif
 */
namespace RTC
{
  /*!
   * @if jp
   * @class CorbaPort
   * @brief RT R|[lg CORBA provider/consumer p Port
   *
   * CorbaPort  RT R|[lgɂāA[U` CORBA IuWF
   * NgT[rXуRV[}񋟂 Port łB
   *
   * <p>
   * RT R|[lǵAPort ă[U` CORBA T[rX
   * 񋟂邱ƂłA RT Service (Provider) ƌĂԁB܂A
   *  RT R|[lg̃T[rX𗘗p邽߂ CORBA IuWFN
   * g̃v[Xz_񋟂邱ƂłA RT Service
   * Consumer ƌĂԁB
   * <p>
   * CorbaPort ͔Cӂ̐ Provider  Consumer Ǘ邱Ƃ
   * APort mڑۂɑΉ Provider  Consumer K؂
   * ֘At邱ƂłB
   * <p>
   * CorbaPort ͒ʏȉ̂悤ɗpB
   *
   * <pre>
   * RTC::CorbaPort m_port0; // Port ̐錾
   *
   * MyService_impl m_mysvc0; //  Port 񋟂 Serivce Provider
   * RTC::CorbaConsumer<YourService> m_cons0; //  Port  Consumer
   *
   * // Service Provider  Port ɓo^
   * m_port0.registerProvider("MyService0", "Generic", m_mysvc0);
   * // Service Consumer  Port ɓo^
   * m_port0.registerConsumer("YourService0", "Generic", m_cons0 );
   *
   * // connect sꂽ
   *
   * m_cons0->your_service_function(); // YourService ̊֐R[
   *
   * // connect ꂽ ʂ̃R|[lgɂ
   * m_cons1->my_service_function(); // MyService ̊֐R[
   * </pre>
   *
   * ̂悤ɁA񋟂 Service Provider  registerProvider() œo
   * ^邱ƂɂÃR|[lg痘p\ɂAAp
   *  Service Consumer  registerConsumer() œo^邱Ƃɂ葼
   * ̃R|[lg Service R|[lgŗp\ɂ邱
   * łB
   *
   * PortInterfaceProfile  Port ɏvoC_̓RV[
   * }C^[tF[XɂĂ̏Lq邽߂̃vt@CłB
   * ڑsc[́ȀɊÂ ConnectorProfile K
   * ɐAڑ\ Port ̂Cӂ̈ɑ΂Ĉ
   * ConnectorProfile ^ Port::connect() ĂяoKvB
   *
   * ȂAPortInterfaceProfile ̃CX^X "*" ͓ȃCX^
   * X\B
   *
   * PROVIDEDȂ킿voC_̃CX^X "*" ̏ꍇ́AڑJ
   * n_ł̓CX^X݂ARV[}̗vɉēI
   * CX^X𐶐^CṽvoC_ł邱Ƃ\B
   * āAڑJn_ł̓CX^X݂͑ȂAڑV[PX
   * ̃C^[tF[XJvZXɂāAvoC_͐
   * CX^XɑΉLqq ConnectorProfile ɓKɐݒ肷
   * ̂ƂB
   *
   * REQUIREDȂ킿RV[}̃CX^X "*" ̏ꍇ́A
   * ̃RV[}̃voC_Ɛڑ\ȃ^CṽRV[}
   * 邱ƂB()
   *
   * ȉ́APortԂ̃C^[tF[Xڑ邽߂ ConnectorProfile 
   * }bsOLq邽߂̃[B
   *
   * PortɕtC^[tF[X̎wq̃tH[}bgȉ̂悤
   * ߂BC^[tF[XɊւvpeBȉ̏ꍇ
   *
   * - RTCCX^X:              rtc_iname
   * - |[g:                       port_name
   * - C^[tF[Xɐ:           if_polarity
   * - C^[tF[X^:           if_tname
   * - C^[tF[XCX^X: if_iname
   * 
   * C^[tF[X̎wqȉ̕񖼏̂Ŏw肷̂ƂB
   *
   * <rtc_iname>.port.<port_name>.<if_polarity>.<if_tname>.<if_iname>
   *
   * PROVIDED()^Ȃ킿voC_̃C^tF[X̃vpeB
   * ̏ꍇA
   *
   * - rtc_iname   = MyComp0
   * - port_name   = myservice
   * - if_polarity = provided
   * - if_tname    = echo_interface
   * - if_iname    = echo_interface2
   *
   * C^[tF[Xwq
   *
   * MyComp0.port.myservice.provided.echo_interface.echo_interface2
   *
   * ̂悤ɋLqB܂AlREQUIRED(v)^Ȃ킿RV[
   * }̃C^[tF[X̃vpeBȉ̏ꍇA
   *
   * - rtc_iname   = YourComp0
   * - port_name   = yourservice
   * - if_polarity = required
   * - if_tname    = hoge_interface
   * - if_iname    = hoge_interface1
   *
   * C^[tF[Xwq́A
   * 
   * YourComp0.port.myservice.required.hoge_interface.hoge_inteface1
   *
   * ̂悤ɋLq邱ƂłB
   * 
   * ȂAœIC^[tF[X̃CX^X̂߂̓ȃ^
   * CṽCX^XLqq
   *
   * - <type_name>*: I^CX^XLqq
   * - <type_name>+: CN^^CX^XLqq
   *
   * `BIC^[tF[XƂ́AڑɃCX^X
   * ^CṽC^[tF[XłB()
   *
   * RV[}vvoC_C^[tF[XLqqɓI^
   * CX^XLqq "<type_name>*" w肳ꂽꍇAvoC_
   * CX^X1VKɐB"<type_name>*" ̋Lqqɂvo
   * C_v n ̃RV[}݂ꍇA炩̗v
   * (Iy[VR[)1 ̃voC_ɂ菈֌W\z
   * (})B
   *
   * <pre>
   * consumer0 ]---<
   * consumer1 ]---<  O----[ provider0
   * consumer2 ]---<
   * </pre>
   *  
   * ɑ΂ARV[}vvoC_C^[tF[XLqq
   * ɃCN^^CX^XLqq "<type_name>+" w
   * ꂽꍇALqq "<type_name>+" ̐voC_̃CX^
   * XIɐBȂ킿A"<type_name>+" ̋Lqqɂvo
   * C_v n ̃RV[}݂ꍇAn ̃voC_
   * ꂼ̗vȉ̂悤Ȋ֌W\zB
   *
   * <pre>
   * consumer0 ]---<  O----[ provider0
   * consumer1 ]---<  O----[ provider1
   * consumer2 ]---<  O----[ provider2
   * </pre>
   *
   *
   * ڑɍۂāAc[ ConnectorProfile::properties ɓK؂ȃC
   * ^[tF[X}bsOwLq邱ƂŁÃ݂voC_/R
   * V[}C^[tF[XRɐڑ邱ƂłBA
   * Ɋւ RTC ̒ɁAقȂCX^XłȂÃCX
   * ^X݂ꍇAC^[tF[XLqq̈Ӑۏ؂ł
   * ̂ŁA̕@ɂڑ͕ۏ؂ȂB
   *
   * ŃC^[tF[XLqqȒP̂߂ <if_desc0>,
   * <if_desc1>, ...  ƂB܂AConnectorProfile::properties 
   * NVList key  value  key: value ̂悤ɋLq̂ƂB
   *
   * ܁A2̃R|[lg̃T[rX|[gڑꍇlB
   * ꂼ̃R|[lg̃T[rX|[gȉ̏ꍇA
   *
   * - rtc_iname: MyComp0        <br>
   *   port_name: mycomp_service <br>
   *   interfaces:
   *   - if_polarity: provided   <br>
   *     if_iname: echo0         <br>
   *     if_tname: Echo
   *   - if_polarity: required   <br>
   *     if_iname: add0          <br>
   *     if_tname: add
   *
   * - rtc_iname: YourComp0        <br>
   *   port_name: yourcomp_service <br>
   *   interfaces:
   *   - if_polarity: required     <br>
   *     if_iname: echo9           <br>
   *     if_tname: Echo
   *   - if_polarity: provided     <br>
   *     if_iname: add9            <br>
   *     if_tname: add
   *
   * <pre>
   *      MyComp0                                 YourComp0
   *     _______ mycomp_service   yourcomp_service ______
   *            |                                 |
   *          |~~~|---O echo0         echo9 >---|~~~|
   *          |   |---< add0          add9  O---|   |
   *           ~T~                               ~T~
   *            |                                 |
   * </pre>
   *
   * MyComp0  echo0 (voC_)  YourComp0  echo9 (RV[})A
   * MyComp0  add0 (RV[})  YourComp0  echo9 (voC_)
   * ꂼ΂ɂĐڑ̂Ɖ肷B̏ꍇA
   * ConnectorProfile ͈ȉ̂悤ɐݒ肷B
   * 
   * <pre>
   * ConnectorProfile:
   *   name: Cӂ̃RlN^
   *   connector_id: 󕶎
   *   ports[]: mycomp_service ̎Q, yourcomp_service ̎Q
   *   properties:
   *     <add0>: <add9>
   *     <echo9>: <echo0>
   * </pre>
   *
   * Aꂼ
   * 
   * <pre>
   * <add0>  MyComp0.port.mycomp_service.required.add.add0
   * <add9>  YourComp0.port.yourcomp_service.provided.add.add9
   * <echo0>  MyComp0.port.mycomp_service.provided.echo.echo0
   * <echo9>  YourComp0.port.yourcomp_service.required.echo.echo9
   * </pre>
   *
   * łBڑvZXɂāAe|[g̃voC_уRV[
   * }́Aꂼȉ̍ƂACorbaPort::publishInterfaces(),
   * CorbaPort::PortsubscribeInterfaces() z֐ɂčsB
   *
   * voC_́ApublishInterfaces() ֐ɂāÃC^[tF[
   * XLqqL[ƂAlIOR̕\L̂
   * ConnectorProfile::properties ɐݒ肷BOƂāÃC^[
   * tF[XLqq͍sƂĂRlN^ɂĂ͈ӂł邽߁A
   * L[1݂Ă͂ȂB
   *
   * [̋̕Lq͖̋@\] ȂAIC^[tF[Xɂ
   * ẮAȉ̎葱ɏ]邱ƂƂȂBpublishInterface()
   * ֐ɂāAICX^XLqq "<type_name>*" ܂́A
   * CN^^CX^XLqq "<type_name>+" ݂
   * 邩ǂ𑖍BICX^XLqq "<type_name>*"
   * ݂ꍇAvoC_̃CX^X1ÃC^[
   * tF[Xwq key ɁAIOR value ɐݒ肷ƂƂɁA
   * ICX^XLqq "<type_name>*"  value Ɋ܂ނׂĂ
   * value ̃C^[tF[XwqAŐC^[tF[X
   * wqɒuB
   * 
   * CN^^CX^XLqq"<type_name>+" ݂
   * ꍇACX^XLqq̐voC_̃CX^X𐶐
   * Aꂼ̃C^[tF[Xwqkey ɁAIOR value 
   * ݒ肷ƂƂɁACN^^CX^XLqq
   * "<type_name>+"  value ܂ނׂĂ value ̃C^[tF[Xw
   * qɑ΂ďɁAŐꂼ̃C^[tF[Xwq
   * uB
   *
   * voC_ subscribeInterfaces() ł͓ɑ͍sȂB
   *
   * RV[}́ApublishInterfaces() ɂĂ͓ɑsȂB
   *
   * AsubscribeInterfaces() ł́A̋Lqq key Ƃ
   * key-value yA ݂邩ǂׁA݂΁A value
   * ɐݒ肳ꂽvoC_̃C^[tF[XwqŎw肳QƂA
   *  ConnectorProfile::properties TARV[}
   * ڑƂĐݒ肷BȂAӐ}IɃRV[}ɃvoC_̎Q
   * ݒ肵Ȃꍇ́A\񕶎 "nil" ܂ "null" ݒ肷
   * ƂB
   *
   * RV[}́A̋Lqq݂ȂꍇA܂̓voC_
   * ̎QƂ Connector::properties ɑ݂ȂꍇARV[}́A
   * ̃CX^Xь^Ɠ̃voC_TA̎QƂ
   * gɐݒ肷B́AOpenRTM-aist-0.4 Ƃ̌݊ێ
   * ߂̃[łA1.0ȍ~ł͐ȂB
   *
   * voC_΃RV[}̑Ή͈ΈłKv͂ȂAvoC_
   * 1 ɑ΂āARV[} nA܂̓RV[} 1 ɑ΂ăvoC
   * _ n ̃P[XBvoC_ 1 ɑ΂āARV[} n 
   * P[Xł́AvoC_̎wqÃRV[}ɑ΂āA
   * L̕@Ŏw肳邱ƂɂABARV[}
   * 1 ɑ΂ăvoC_ n ̃P[Xł́ARV[}wq key ɑ
   * āÃvoC_̎wqJ}؂ŗ񋓂`ƂȂ
   * ̂ƂB
   *
   * ȂAC^[tF[X̑Ή֌W̌w肷IvVƂāA
   * ȉ̃IvVw肷邱ƂłB
   *
   * port.connection.strictness: strict, best_effort 
   *
   * strict: ׂẴRV[}Ɏw肵QƂ݂Ai[C
   *         OɂRV[}ɓK؂ɃZbgłꍇɂ̂ Port
   *         Ԃ̐ڑmB
   *
   * best_effort: i[COɎsꍇłAG[ԂƂ
   *          Port Ԃ̐ڑmB
   *
   * @since 0.4.0
   *
   * @else
   * @class CorbaPort
   * @brief RT Conponent CORBA service/consumer Port
   *
   * CorbaPort is an implementation of the Port of RT-Component's that provides
   * user-defined CORBA Object Service and Consumer.
   * <p>
   * RT-Component can provide user-defined CORBA serivces, which is called
   * RT-Serivce (Provider), through the Ports.
   * RT-Component can also provide place-holder, which is called RT-Serivce
   * Consumer, to use other RT-Component's service.
   * <p>
   * The CorbaPort can manage any number of Providers and Consumers, can
   * associate Consumers with correspondent Providers when establishing
   * connection among Ports.
   * <p>
   * Usually, CorbaPort is used like the following.
   *
   * <pre>
   * RTC::CorbaPort m_port0; // declaration of Port
   *
   * MyService_impl m_mysvc0; // Serivce Provider that is provided by the Port
   * RTC::CorbaConsumer<YourService> m_cons0; // Consumer of the Port
   *
   * // register Service Provider to the Port
   * m_port0.registerProvider("MyService0", "Generic", m_mysvc0);
   * // register Service Consumer to the Port
   * m_port0.registerConsumer("YourService0", "Generic", m_cons0 );
   *
   * // after connect established
   *
   * m_cons0->your_service_function(); // call a YourService's function
   *
   * // in another component that is connected with the Port
   * m_cons1->my_service_function(); // call a MyService's function
   * </pre>
   *
   * Registering Service Provider by registerProvider(), it can be
   * used from other RT-Components.  Registering Service Consumer by
   * registerConsumer(), other RT-Component's services can be used
   * through the consumer object.
   *
   * PortInterfaceProfile is a one of the profile information to store
   * Provider interface and Consumer interface information. Tools or
   * other RTCs should call one of the Port::connect() with an
   * appropriate ConnectorProfile.
   *
   * In addition, the instance name "*" declares a special type of instance.
   *
   * When the name of the PROVIDED type interface that is the provider
   * interface is "*", Provider interface's instance does not exist at
   * the beginning of connection sequence.  The instances will be
   * created dynamically according to the consumer interface
   * requirement at the connection sequence.  Although the instance
   * name does not exist at the beginning of connection sequence, the
   * created providers shall publish its references to the
   * ConnectorProfile with interface descriptor adequately in the
   * interface publisher phase of the connection sequence.
   *
   * If REQUIRED interface name that is Consumer interface name is
   * "*", it shows that one Consumer interface is able to connect with
   * multiple Provider interfaces. (This feature is not implemented.)
   * 
   * The following describes the rules that specify interface
   * connection between ports.
   *
   * The descriptor format of interfaces associated with Ports is
   * declared as follows. Now some of interface properties are assumed
   * as the followings.
   *
   * - RTC instance name:              rtc_iname
   * - Port name:                      port_name
   * - Interface polarity:             if_polarity
   * - Interface type name:            if_tname
   * - INterface instance name:        if_iname
   *
   * The interface descriptors shall be declared as follows.
   *
   * <rtc_iname>.port.<port_name>.<if_polarity>.<if_tname>.<if_iname>
   *
   * When PROVIDED that is Provider interface properties are the followings,
   *
   * - rtc_iname   = MyComp0
   * - port_name   = myservice
   * - if_polarity = provided
   * - if_tname    = echo_interface
   * - if_iname    = echo_interface2
   * the interface descriptor is here.
   *
   * MyComp0.port.myservice.provided.echo_interface.echo_interface2
   *
   * And, when REQUIRED that is Consumer interfaces properties are the
   * followings,
   *
   * - rtc_iname   = YourComp0
   * - port_name   = yourservice
   * - if_polarity = required
   * - if_tname    = hoge_interface
   * - if_iname    = hoge_interface1
   *
   * interface descriptor is as follows. 
   *
   * YourComp0.port.myservice.required.hoge_interface.hoge_inteface1
   *
   * Specific instance name descriptors that are dynamically generated
   * at the connection time are defined here.
   *
   * - <type_name>*: "Dynamically generated" instance descriptor.
   * - <type_name>+: "Incrementally generated" instance descriptor.
   *
   * When the "Dynamically generated" instance descriptor:
   * "<type_name>*" is specified as interface descriptor that is
   * required by consumers, the provider will generate a instance. If
   * n consumers who demand a provider by the "<type_name>" descriptor
   * exist, the following relation which processes the call from these
   * consumers by one provider will be established.
   *
   * <pre>
   * consumer0 ]---<
   * consumer1 ]---<  O----[ provider0
   * consumer2 ]---<
   * </pre>
   *  
   * On the other hand, when incremental generated type instance name
   * descriptor "<type_name>+" is specified as the provider interface
   * descriptor whom consumers demand, provider's instances are
   * dynamically generated for the number of the descriptors
   * "<type_name>+". When n consumers who demand a provider by the
   * descriptor "<type_name>+" exist the following relations in which
   * n providers process each call from the consumers will be
   * established.
   *
   * <pre>
   * consumer0 ]---<  O----[ provider0
   * consumer1 ]---<  O----[ provider1
   * consumer2 ]---<  O----[ provider2
   * </pre>
   *
   *
   * Describing the appropriate interface mapping specification in the
   * ConnectorProfile::properties, selective connections between
   * providers/consumers interface can be established at the time of
   * connection. However, when different RTC instances of the same
   * instance name exist in a connection, since an interface
   * descriptor uniqueness cannot be guaranteed, this connection
   * mapping rules cannot be used.
   *
   * Here, assume that an interface descriptor is given as <if_desc0>,
   * <if_desc1>, .... And assume that the key and the value of NVList
   * in ConnectorProfile::properties are given as "key: value".
   *
   * Now the case where the service ports of two components are
   * connected is considered. When the service port of each component
   * is the following,
   * 
   * - rtc_iname: MyComp0          <br>
   *   port_name: mycomp_service   <br>
   *   interfaces:
   *   - if_polarity: provided     <br>
   *     if_iname: echo0           <br>
   *     if_tname: Echo
   *   - if_polarity: required     <br>
   *     if_iname: add0            <br>
   *     if_tname: add
   *
   * - rtc_iname: YourComp0        <br>
   *   port_name: yourcomp_service <br>
   *   interfaces:
   *   - if_polarity: required     <br>
   *     if_iname: echo9           <br>
   *     if_tname: Echo
   *   - if_polarity: provided     <br>
   *     if_iname: add9            <br>
   *     if_tname: add
   *
   *
   * <pre>
   *      MyComp0                                 YourComp0
   *     _______ mycomp_service   yourcomp_service ______
   *            |                                 |
   *          |~~~|---O echo0         echo9 >---|~~~|
   *          |   |---< add0          add9  O---|   |
   *           ~T~                               ~T~
   *            |                                 |
   * </pre>
   * 
   *
   *
   * Assume that connection between echo0 (provider) of MyComp0
   * component and echo9 (consumer) of YourComp0 component, and add0
   * (consumer) of MyComp0 and add0 (provider) of YourComp0 is
   * established.  In this case, ConnectorProfile is set up as
   * follows.
   * 
   * <pre>
   * ConnectorProfile:
   *   name: any connector name
   *   connector_id: empty string
   *   ports[]: mycomp_service's reference, yourcomp_service's reference
   *   properties:
   *     <add0>: <add9>
   *     <echo9>: <echo0>
   * </pre>
   *
   * Please note that <add0>, <add9>, <echo0> and <echo9> are the following.
   * 
   * <pre>
   * <add0> is MyComp0.port.mycomp_service.required.add.add0
   * <add9> is YourComp0.port.yourcomp_service.provided.add.add9
   * <echo0> is MyComp0.port.mycomp_service.provided.echo.echo0
   * <echo9> is YourComp0.port.yourcomp_service.required.echo.echo9
   * </pre>
   *
   * In the connection process, the provider and the consumer of each
   * port carries out the following process respectively in the
   * virtual functions such as CorbaPort::publishInterfaces() and
   * CorbaPort::subscribeInerfaces().
   * 
   * A provider sets its IOR string as a value and its interface
   * descriptor as a key in the ConnectorProfile::properties in a
   * publishInterfaces() function. Since this interface descriptor's
   * uniqueness is guaranteed in the current connector, the key of
   * NameValue in the ConnectorProfile::properties is unique.
   *
   *
   * [This functionalities are not implemented] The dynamically
   * generated provider is processed according to the following
   * procedure. The publishInterface() function scans dynamic instance
   * descriptors such as "<type_name>*" and "<type_name>+" in the
   * ConnectorProfile::properties. When the dynamic generation
   * instance descriptor "<tupe_name>*" exists, one instance of
   * provider is generated, and its descriptor and its IOR string are
   * set to ConnectorProfile::properties as the key and the value
   * respectively. Simultaneously, in the
   * ConnectorProfile::properties, all the instance descriptor with
   * the dynamic generation instance name "<type_name>*" will be
   * replaced with newly generated instance descriptor.
   *
   * When the incremental dynamic generation instance descriptor
   * exists, providers are generated for the number of the
   * descriptors, and its descriptor and its IOR string are set to
   * ConnectorProfile::properties as the key and the value
   * respectively. Simultaneously, in the
   * ConnectorProfile::properties, all the instance descriptor with
   * the dynamic generation instance name "<type_name>+" will be
   * replaced with newly generated instance descriptor.
   *
   * The providers do not perform particular operation in
   * subscribeInterfaces() function.
   *
   *
   * The consumers do not perform particular operation in
   * publisherInterfaces() function.
   *
   * On the other hand, a consumer searches a key-value pair with the
   * key of consumer interface descriptor, and if the pair exists, it
   * obtains provider's descriptor from the value. The consumer
   * searches again a key-value pair with the key of provider
   * interface descriptor, and it obtains provider's reference and the
   * reference is set as the consumer's service object. In addition,
   * reserved string "nil" or "null" are used not to set specific
   * provider.
   *
   * If consumer's interface descriptors does not exists in the
   * ConnectorProfile::properties, the consumer searches a provider
   * with same type name and instance name, and its reference is set
   * to the consumer. This rule is for only backward compatibility,
   * and it is not recommended from version 1.0.
   *
   * The correspondence of a provider versus a consumer does not need
   * to be one to one, and the case of one provider to n-consumers and
   * the case of m-providers to one consumer are allowed. The one
   * provider to n-consumers case can be realized by the above
   * mentioned methods. The one consumer to m-provider case can be
   * specified to set the consumer descriptor and comma-separated
   * provider descriptors into the key and the value respectively.
   *
   * The following option is available to specify the strictness of
   * interfaces connection.
   *
   * port.connection.strictness: strict, best_effort
   *
   * strict: The connection is established, if only all the specified
   *         consumers are set appropriate references and narrowed
   *         successfully.  
   *
   * best_effort: The connection is established without any errors,
   *         even if appropriate reference does not exist or reference
   *         narrowing fails.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class CorbaPort
    : public PortBase
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * @param name Port ̖O
     *
     * @else
     *
     * @brief Constructor
     *
     * @param name The name of Port 
     *
     * @endif
     */
    CorbaPort(const char* name);
    
    /*!
     * @if jp
     *
     * @brief zfXgN^
     *
     * @else
     *
     * @brief Virtual destructor
     *
     * @endif
     */
    virtual ~CorbaPort(void);
    
    /*!
     * @if jp
     * @brief vpeB̏
     *
     * OutPort̃vpeB
     *
     * @else
     *
     * @brief Initializing properties
     *
     * This operation initializes outport's properties
     *
     * @endif
     */
    void init(coil::Properties& prop);

    /*!
     * @if jp
     *
     * @brief Provider o^
     *
     *  Port ɂĒ񋟂T[og Port ɑ΂ēo^B
     * T[oǵAŗ^ instance_name, type_name A
     * T[ogg̃CX^Xу^CvƂāAT[og
     * ֘AtB
     *
     * @param instance_name T[og̃CX^X
     * @param type_name T[og̃^Cv
     * @param provider CORBA T[og
     *
     * @return ɓ instance_name o^Ă false ԂB
     *
     * @else
     *
     * @brief Register the provider
     *
     * This operation registers a servant, which is provided in this Port,
     * to the Port. The servant is associated with "instance_name" and
     * "type_name" as the instance name of the servant and as the type name
     * of the servant.
     *
     * @param instance_name Instance name of servant
     * @param type_name Type name of the servant
     * @param provider CORBA servant
     *
     * @return Return false if the same name of instance_name is already 
     *         registered.
     *
     * @endif
     */
    bool registerProvider(const char* instance_name, const char* type_name,
                          PortableServer::RefCountServantBase& provider);
    
    /*!
     * @if jp
     *
     * @brief Consumer o^
     *
     *  Port vT[rX̃v[Xz_łRV[}
     * (Consumer) o^BConsumer ֘AtT[rX̃CX
     * ^Xу^CvƂāA instance_name, type_name 
     *  Consumer g^邱ƂɂAł炪֘At
     * BPort Ԃ̐ڑ (connect)  ɂ́AsubscribeInterfaces() ŏq
     * ׂĂ郋[ɊÂAProvider Interface ̎QƂI
     * Consumer ɃZbgB
     *
     * @param instance_name Consumer vT[rX̃CX^X
     * @param type_name Consumer vT[rX̃^Cv
     * @param consumer CORBA T[rXRV[}
     *
     * @return ɓ instance_name o^Ă false ԂB
     *
     * @else
     *
     * @brief Register the consumer
     *
     * This operation registers a consumer, which is a service
     * placeholder this port requires. These are associated internally
     * with specified instance_name, type_name and Consumer itself to
     * the argument as service's instance name and its type name
     * associated with Consumer.  The service Provider interface'
     * references will be set automatically to the Consumer Interface
     * object when connections are established, according to the rules
     * that are described at the subscribeInterface() function's
     * documentation.
     *
     * @param instance_name Instance name of the service Consumer requires
     * @param type_name Type name of the service Consumer requires
     * @param consumer CORBA service consumer
     *
     * @return False would be returned if the same instance_name was registered
     *
     * @endif
     */
    bool registerConsumer(const char* instance_name, const char* type_name,
                          CorbaConsumerBase& consumer);
    
  protected:
    /*!
     * @if jp
     *
     * @brief Provider Interface J
     *
     *  Port L Provider C^[tF[XɊւ
     * ConnectorProfile::properties ɑ Port ɑ΂ČJB
     * ARTC̃CX^X̏񂪈ȉ̒ʂłƂāA
     *
     * - RTCCX^X:              rtc_iname
     * - |[g:                       port_name
     * - C^[tF[Xɐ:           if_polarity
     * - C^[tF[X^:           if_tname
     * - C^[tF[XCX^X: if_iname
     *
     * NameValue ^ ConnectorProfile::properties  name  value Ƃ
     * ȉ̂̂i[B
     *
     * - name
     *   <rtc_iname>.port.<port_name>.provided.<if_tname>.<if_iname>
     * - value
     *   Provider C^[tF[X IOR  
     * 
     * ȂAo[WƂ̌݊̂߈ȉ̕\L NameValue 
     * Ɋi[邪Ão[Wł͍폜\B
     * 
     * - name
     *   port.<if_tname>.<if_iname>
     * - value
     *   Provider C^[tF[X IOR 
     *
     * ̒l ConnectorProfile::properties Ɋi[Ã|[gɑ΂
     * `BB Port ł̃C^[tF[Xgp Consumer 
     * ݂΁AConnectorProfile 炱̃L[IuWFNgt@X
     * 擾炩̌`ŎgpB
     *
     * @param connector_profile RlN^vt@C
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Publish information about interfaces
     *
     * This operation publishes Provider interfaces information, which
     * is owned by this port, to the other Ports via
     * ConnectorProfile::properties.
     * Now it is assumed RTC instance name and other information is as follows,
     *
     * - RTC instance name:              rtc_iname
     * - Port name:                      port_name
     * - Interface polarity:             if_polarity
     * - Interface type name:            if_tname
     * - Interface instance name:        if_iname
     *
     * the following values are stored as the "name" and the "value"
     * of the NameValue typee element in ConnectorProfile::properties.
     *
     * - name
     *   <rtc_iname>.port.<port_name>.provided.<if_tname>.<if_iname>
     * - value
     *   IOR string value of interface reference
     * 
     * In addition, although the following NameValue values are also
     * stored for the backward compatibility, this will be deleted in
     * the future version.
     *
     * - name
     *   port.<if_tname>.<if_iname>
     * - value
     *   IOR string value of interface reference
     *
     * These values are stored in the ConnectorProfile::properties and
     * are propagated to the other Ports. If the Consumer interface
     * exists that requires this Provider interface, it will retrieve
     * reference from the ConnectorProfile and utilize it.
     *
     * @param connector_profile Connector profile
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t
    publishInterfaces(ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     *
     * @brief Provider Interface 擾
     *
     *  PortL Consumer Interface ɓK Provider
     * Interface ɊւConnectorProfile::properties 璊o
     * Consumer Interface ɃIuWFNgQƂZbgB
     *
     * ARTC ̃CX^X Consumer Interface ̏񂪈ȉ̂
     * łƉ肷ƁA
     *
     * - RTCCX^X:              rtc_iname
     * - |[g:                       port_name
     * - C^[tF[Xɐ:           if_polarity
     * - C^[tF[X^:           if_tname
     * - C^[tF[XCX^X: if_iname
     *
     *  Consumer Interface \C^[tF[Xwq͈ȉ̂悤
     * ɕ\B
     *
     * <rtc_iname>.port.<port_name>.required.<if_tname>.<if_iname>
     *
     * ̊֐́A܂ ConnectorProfile::properties ɏLC^[
     * tF[XwqL[ƂĊi[Ă Provider Interface w
     * qToBɁA Provider Interface wqL[Ƃ
     * i[Ă Provider Interface ̎QƂ\ IOR 擾
     * AConsumer Interface ɃZbgB
     *
     * AɁAProvider  prov(n), ̎QƂIOR(n)  Consumer
     * cons(n) ̂悤ɋLqA炷ׂẴC^[tF[X̌^
     * łAConnectorProfile Ɉȉ̒lݒ肳ĂƂB
     *
     * <pre>
     * ConnectorProfile::properties =
     * {
     *   prov0: IOR0,
     *   prov1: IOR1,
     *   prov2: IOR2,
     *   cons0: prov2,
     *   cons1: prov1,
     *   cons2: prov0
     * }
     * </pre>
     *
     * ̂ƂAcons(0..2) ɂ͂ꂼAQƂȉ̂悤ɃZbgB
     *
     * <pre>
     *   cons0 = IOR2
     *   cons1 = IOR1
     *   cons2 = IOR0
     * </pre>
     *
     * ȂAo[WƂ̌݊̂߁A
     * ConnectorProfile::properties  Consumer Interface L[Ƃ
     * lZbgĂȂꍇłÃ[KpB
     *
     * A Consumer Interface 
     *
     * <pre>
     *  PortInterfaceProfile
     *  {
     *    instance_name = "PA10_0";
     *    type_name     = "Manipulator";
     *    polarity      = REQUIRED;
     *  }
     * </pre>
     *
     * Ƃēo^Ă΁A Port 
     *
     * <pre>
     *  PortInterfaceProfile
     *  {
     *    instance_name = "PA10_0";
     *    type_name     = "Manipulator";
     *    polarity      = PROVIDED;
     *  }
     * </pre> 
     *
     * Ƃēo^Ă Serivce Provider ̃IuWFNgQƂTA
     * Consumer ɃZbgBۂɂ́AConnectorProfile::properties 
     *
     * <pre>
     * NameValue = { "port.Manipulator.PA10_0": <Object reference> }
     * </pre>
     *
     * Ƃēo^Ă NameValue TÃIuWFNgQƂ
     * Consumer ɃZbgB
     *
     * @param connector_profile RlN^vt@C
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief Subscribe to interface
     *
     * Retrieve information associated with Provider matches Consumer
     * owned by this port and set the object reference to Consumer.
     *
     * Now, Consumer is registered as the following:
     * <pre>
     *  PortInterfaceProfile
     *  {
     *    instance_name = "PA10_0";
     *    type_name     = "Manipulator";
     *    polarity      = REQUIRED;
     *  }
     * </pre>
     * Find the object reference of Serivce Provider that is registered as
     * the following of other ports:
     * <pre>
     *  PortInterfaceProfile
     *  {
     *    instance_name = "PA10_0";
     *    type_name     = "Manipulator";
     *    polarity      = PROVIDED;
     *  }
     * </pre> 
     * and set to Consumer.
     * In fact, find NameValue that is registered as the following to 
     * ConnectorProfile::properties:
     * <pre>
     * NameValue = { "port.Manipulator.PA10_0": <Object reference> }
     * </pre>
     * and set the object reference to Consumer.
     *
     * @param connector_profile Connector profile
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t
    subscribeInterfaces(const ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     *
     * @brief Interface ւ̐ڑ
     *
     * ^ꂽ ConnectorProfile Ɋ֘A Consumer ɃZbgꂽ
     * ׂĂ Object ڑB
     *
     * @param connector_profile RlN^vt@C
     *
     * @else
     *
     * @brief Unsubscribe interfaces
     *
     * Release all Objects that was set in Consumer associated with the given 
     * ConnectorProfile.
     *
     * @param connector_profile Connector profile
     *
     * @endif
     */
    virtual void
    unsubscribeInterfaces(const ConnectorProfile& connector_profile);

    //============================================================
    // Local operations
    //============================================================
    /*!
     * @if jp
     *
     * @brief Port ̑SẴC^[tF[X activates 
     *
     * Port ɓo^ĂSẴC^[tF[X activate B
     *
     * @else
     *
     * @brief Activate all Port interfaces
     *
     * This operation activate all interfaces that is registered in the
     * ports.
     *
     * @endif
     */
    virtual void activateInterfaces();

    /*!
     * @if jp
     *
     * @brief SĂ Port ̃C^[tF[X deactivates 
     *
     * Port ɓo^ĂSẴC^[tF[X deactivate B
     *
     * @else
     *
     * @brief Deactivate all Port interfaces
     *
     * This operation deactivate all interfaces that is registered in the
     * ports.
     *
     * @endif
     */
    virtual void deactivateInterfaces();

  protected:
    /*!
     * @if jp
     * @brief vpeB
     * @else
     * @brief Properties
     * @endif
     */
    coil::Properties m_properties;
    
  private:
    class CorbaConsumerHolder;
    virtual bool findProvider(const NVList& nv, CorbaConsumerHolder& cons);
    virtual bool findProviderOld(const NVList&nv, CorbaConsumerHolder& cons);

  private:
    /*!
     * @if jp
     * @class CorbaProviderHolder
     * @brief Provider ̏i[\
     *
     * CORBA Provider ̃z_NX
     *
     * @else
     * @class CorbaProviderHolder
     * @brief The structure to be stored Provider information.
     *
     * CORBA Provider holder class
     *
     * @endif
     */
    class CorbaProviderHolder
    {
    public:
      CorbaProviderHolder(const char* type_name,
                          const char* instance_name,
                          PortableServer::RefCountServantBase* servant)
        : m_typeName(type_name),
          m_instanceName(instance_name),
          m_servant(servant),
          m_ior()
      {  
        m_oid = Manager::instance().getPOA()->servant_to_id(m_servant);
        try
          {
            Manager::instance().
              getPOA()->activate_object_with_id(m_oid, m_servant);
          }
        catch(const ::PortableServer::POA::ServantAlreadyActive &)
          {
            ;
          }
        catch(const ::PortableServer::POA::ObjectAlreadyActive &)
          {
            ;
          }
        CORBA::Object_var obj;
        obj = Manager::instance().getPOA()->id_to_reference(m_oid);
        CORBA::ORB_ptr orb = Manager::instance().getORB();
        CORBA::String_var ior_var = orb->object_to_string(obj);
        m_ior = ior_var;
        deactivate();
      }

      std::string instanceName() { return m_instanceName; }
      std::string typeName() { return m_typeName; }
      std::string ior() { return m_ior; }
      std::string descriptor() { return m_typeName + "." + m_instanceName; }

      void activate()
      {
        try
          {
            Manager::instance().
              getPOA()->activate_object_with_id(m_oid, m_servant);
          }
        catch(const ::PortableServer::POA::ServantAlreadyActive &)
          {
            ; // do nothing
          }
        catch(const ::PortableServer::POA::ObjectAlreadyActive &)
          {
            ; // do nothing
          }
      }
      void deactivate()
      {
        try
          {
            Manager::instance().getPOA()->deactivate_object(m_oid);
          }
        catch(const ::PortableServer::POA::ObjectNotActive&)
          {
            ; // do nothing
          }
      }
    private:
      std::string m_typeName;
      std::string m_instanceName;
      PortableServer::RefCountServantBase* m_servant;
      PortableServer::ObjectId_var m_oid;
      std::string m_ior;
    };

    /*!
     * @if jp
     * @brief Provider ̏i[ vector
     * @else
     * @brief vector to stored Providers' information
     * @endif
     */
    typedef std::vector<CorbaProviderHolder> CorbaProviderList;
    CorbaProviderList m_providers;

    /*!
     * @if jp
     * @brief Consumer ̏i[\
     * @else
     * @brief The structure to be stored Consumer information.
     * @endif
     */
    class CorbaConsumerHolder
    {
    public:
      CorbaConsumerHolder(const char* type_name,
                          const char* instance_name,
                          CorbaConsumerBase* consumer)
        : m_typeName(type_name),
          m_instanceName(instance_name),
          m_consumer(consumer)
      {
      }
      std::string instanceName() { return m_instanceName; }
      std::string typeName() { return m_typeName; }
      std::string descriptor() { return m_typeName + "." + m_instanceName; }

      bool setObject(const char* ior)
      {
        CORBA::ORB_ptr orb = ::RTC::Manager::instance().getORB();
        CORBA::Object_var obj = orb->string_to_object(ior);
        if (CORBA::is_nil(obj))
          {
            return false;
          }

        return m_consumer->setObject(obj.in());
      }
      void releaseObject()
      {
        m_consumer->releaseObject();
      }

    private:
      std::string m_typeName;
      std::string m_instanceName;
      CorbaConsumerBase* m_consumer;
    };
    typedef std::vector<CorbaConsumerHolder> CorbaConsumerList;
    CorbaConsumerList m_consumers;
    
    // functors
    /*!
     * @if jp
     * @brief Consumer ̃IuWFNg邽߂ Functor
     * @else
     * @brief Functor to release Consumer's object
     * @endif
     */
    struct unsubscribe
    {
      unsubscribe(CorbaConsumerList& consumers)
        : m_consumers(consumers)
      {
      }
      
      void operator()(const SDOPackage::NameValue& nv)
      {
        for (CorbaConsumerList::iterator it(m_consumers.begin());
             it != m_consumers.end(); ++it)
          {
            std::string name(nv.name);
            if (it->descriptor() == (const char*)nv.name)
              {
                it->releaseObject();
              }
          }
      }
      CorbaConsumerList& m_consumers;
    };
  };
};
#endif // RTC_CORBAPORT_H
