// -*- C++ -*-
/*!
 * @file Factory.h
 * @brief RT-Component factory class
 * @date $Date: 2007-12-31 03:08:03 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2003-2008
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: Factory.h 1569 2009-11-12 14:36:47Z n-ando $
 *
 */

#ifndef RTC_FACTORY_H
#define RTC_FACTORY_H

#include <coil/Properties.h>
//#include <rtm/RTObject.h>
#include <rtm/NumberingPolicy.h>


namespace RTC 
{
  class RTObject_impl;
  class Manager;
  
  typedef RTObject_impl* (*RtcNewFunc)(Manager* manager);
  typedef void (*RtcDeleteFunc)(RTObject_impl* rtc);
  
  /*!
   * @if jp
   *
   * @brief RTR|[lgpev[g֐
   * 
   * RTR|[lg̃CX^X𐶐邽߂̃ev[g֐B
   * RTR|[lgǗp}l[WĂяoB
   * ۂɂ͊eR|[lg̃RXgN^ĂяoB
   * \<_New\>ŐΏRTR|[lǧ^w肷B
   *
   * @param manager }l[WIuWFNg
   *
   * @return  RTR|[lg CX^X
   * 
   * @else
   * @brief Template function to create RT-Components
   * 
   * This is the template function to create RT-Component's instances.
   * This is invoked from RT-Components manager.
   * Actually, each component's constructor is invoked.
   * Specify the type of the target RT-Components for creation by \<_New\>.
   *
   * @param manager Manager object
   *
   * @return Created RT-Component's instances
   *
   * @endif
   */
  template <class _New>
  RTObject_impl* Create(Manager* manager)
  {
    return new _New(manager);
  }
  
  /*!
   * @if jp
   *
   * @brief RTR|[lgjpev[g֐
   * 
   * RTR|[lg̃CX^Xj邽߂̃ev[g֐B
   * \<_Delete\>ɂĔjΏRTR|[lǧ^w肷B
   *
   * @param rtc jΏRTR|[lg̃CX^X
   *
   * @else
   *
   * @brief Template function to destroy RT-Components
   * 
   * This is the template function to destroy RT-Component's instances.
   * Specify the type of the target RT-Components for destroy by \<_Delete\>.
   *
   * @param rtc The target RT-Component's instances for destruction
   *
   * @endif
   */
  template <class _Delete>
  void Delete(RTObject_impl* rtc)
  {
    delete rtc;
  }
  
  /*!
   * @if jp
   *
   * @class FactoryBase
   * @brief FactoryBase NX
   * 
   * R|[lgt@Ng̊NXB
   *
   * @since 0.2.0
   *
   * @else
   *
   * @class FactoryBase
   * @brief FactoryBase base class
   *
   * This is a base class for RT-Component factory.
   *
   * @since 0.2.0
   *
   * @endif
   */
  class FactoryBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * RXgN^B
     *
     * @param profile R|[lg̃vt@C
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor.
     *
     * @param profile Component profile
     *
     * @endif
     */
    FactoryBase(const coil::Properties& profile);
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~FactoryBase(void);
    
    /*!
     * @if jp
     *
     * @brief R|[lg̐
     *
     * RT-Component ̃CX^X𐶐邽߂̏z֐B
     *
     * @param mgr }l[WIuWFNg
     *
     * @return R|[lg
     *
     * @else
     *
     * @brief Create components
     *
     * Pure virtual function to create RT-Component's instances
     *
     * @param mgr Manager object
     *
     * @return Created RT-Components
     *
     * @endif
     */
    virtual RTObject_impl* create(Manager* mgr) = 0;
    
    /*!
     * @if jp
     *
     * @brief R|[lg̔j
     *
     * RT-Component ̃CX^Xj邽߂̏z֐B
     *
     * @param comp jΏ RTR|[lg
     *
     * @else
     *
     * @brief Destroy components
     *
     * Pure virtual function to destroy RT-Component's instances
     *
     * @param comp The target RT-Component for destruction
     *
     * @endif
     */
    virtual void destroy(RTObject_impl* comp) = 0;
    
    /*!
     * @if jp
     *
     * @brief R|[lgvt@C̎擾
     *
     * R|[lg̃vt@C擾
     *
     * @return R|[lg̃vt@C
     *
     * @else
     *
     * @brief Get the component profile
     *
     * Get the component profile.
     *
     * @return The component profile
     *
     * @endif
     */
    virtual coil::Properties& profile();
    
    /*!
     * @if jp
     *
     * @brief ݂̃CX^X̎擾
     *
     * R|[lǧ݂̃CX^X擾B
     *
     * @return R|[lg̃CX^X
     *
     * @else
     *
     * @brief Get the number of current instances
     *
     * Get the number of current RT-Component's instances.
     *
     * @return Number of RT-Component's instances
     *
     * @endif
     */
    virtual int number();
    
  protected:
    /*!
     * @if jp
     * @brief R|[lg̃vt@C
     * @else
     * @brief Component profile
     * @endif
     */
    coil::Properties m_Profile;
    
    /*!
     * @if jp
     * @brief ݂̃CX^X
     * @else
     * @brief Number of current RT-Component's instances.
     * @endif
     */
    int m_Number;
  };
  
  /*!
   * @if jp
   * @class FactoryCXX
   * @brief FactoryCXX NX
   * 
   * C++pR|[lgt@NgNXB
   *
   * @since 0.2.0
   *
   *
   * @else
   *
   * @class FactoryCXX
   * @brief FactoryCXX class
   *
   * RT-Component factory class for C++.
   *
   * @since 0.2.0
   *
   * @endif
   */
  class FactoryCXX
    : public FactoryBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * RXgN^B
     * vt@CA֐ւ̃|C^Aj֐ւ̃|C^A
     * R|[lg̖|V[ɎA
     * C++ ŎꂽR|[lg̃t@NgNX𐶐B
     *
     * @param profile R|[lg̃vt@C
     * @param new_func R|[lg̐֐ւ̃|C^
     * @param delete_func R|[lg̔j֐ւ̃|C^
     * @param policy R|[lg̖|V[
     * (ftHgl:DefaultNumberingPolicy)
     *
     * @else
     *
     * @brief Constructor.
     *
     * Constructor.
     * Create component factory class with three arguments:
     * component profile, function pointer to object create function and
     * object destroy function.
     *
     * @param profile Component profile
     * @param new_func Pointer to component create function
     * @param delete_func Pointer to component destroy function
     * @param policy The naming policy at component creation
     * (The default value:DefaultNumberingPolicy)
     *
     * @endif
     */
    FactoryCXX(const coil::Properties& profile,
	       RtcNewFunc new_func,
	       RtcDeleteFunc delete_func,
	       NumberingPolicy* policy = new DefaultNumberingPolicy());
    
    virtual ~FactoryCXX()
    {
      delete m_policy;
    }

    /*!
     * @if jp
     *
     * @brief R|[lg̐
     *
     * RT-Component ̃CX^X𐶐B
     *
     * @param mgr }l[WIuWFNg
     *
     * @return R|[lg
     *
     * @else
     *
     * @brief Create RT-Components
     *
     * Create RT-Component's instances
     *
     * @param mgr Manager object
     *
     * @return Created RT-Components
     *
     * @endif
     */
    virtual RTObject_impl* create(Manager* mgr);
    
    /*!
     * @if jp
     *
     * @brief R|[lg̔j
     *
     * RT-Component ̃CX^XjB
     *
     * @param comp jΏ RT-Component
     *
     * @else
     *
     * @brief Destroy RT-Components
     *
     * Destroy RT-Component's instances
     *
     * @param comp The target RT-Component for destruction
     *
     * @endif
     */
    virtual void destroy(RTObject_impl* comp);
    
  protected:
    /*!
     * @if jp
     * @brief R|[lgIuWFNg֐ւ̃|C^
     * @else
     * @brief The pointer to component object create function
     * @endif
     */
    RtcNewFunc m_New;
    
    /*!
     * @if jp
     * @brief R|[lgIuWFNgj֐ւ̃|C^
     * @else
     * @brief The pointer to component object destroy function
     * @endif
     */
    RtcDeleteFunc m_Delete;
    
    /*!
     * @if jp
     * @brief R|[lg̖|V[
     * @else
     * @brief The naming policy on creating the components
     * @endif
     */
    NumberingPolicy* m_policy;
  };
};
#endif // RTC_FACTORY_H
