// -*- C++ -*-
/*!
 * @file InPortBase.h
 * @brief RTC::Port implementation for InPort
 * @date $Date: 2008-01-13 15:06:40 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: InPortBase.h 1842 2010-02-01 03:16:07Z fsi-katami $
 *
 */

#ifndef RTC_INPORTBASE_H
#define RTC_INPORTBASE_H

#include <rtm/PortBase.h>
#include <rtm/DataPortStatus.h>
#include <rtm/CdrBufferBase.h>
#include <rtm/ConnectorListener.h>

/*!
 * @if jp
 * @namespace RTC
 *
 * @brief RTR|[lg
 *
 * @else
 *
 * @namespace RTC
 *
 * @brief RT-Component
 *
 * @endif
 */
namespace RTC
{
  class InPortProvider;
  class OutPortConsumer;
  class InPortConnector;

  /*!
   * @if jp
   * @class InPortBase
   * @brief InPort p Port
   *
   * f[^̓|[g̎NXB
   *
   * @since 0.4.0
   *
   * @else
   * @class InPortBase
   * @brief Port for InPort
   *
   * This is an implementation class for the data input port.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class InPortBase
    : public PortBase, public DataPortStatus
  {
  public:
    DATAPORTSTATUS_ENUM

    typedef std::vector<InPortConnector*> ConnectorList;

    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param name |[g
     * @param data_type f[^^Cv
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @param name Port name
     * @param data_type Data type
     *
     * @endif
     */
    InPortBase(const char* name, const char* data_type);
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~InPortBase(void);

    /*!
     * @if jp
     * @brief vpeB̏
     *
     * w肳ꂽvpeBŏB
     *
     * @param prop ݒ肷vpeB
     * @else
     * @brief Initializing properties
     *
     * This method initializes the port in the specified property. 
     *
     * @param prop Property for setting ports
     * @endif
     */
    void init(coil::Properties& prop);

    /*!
     * @if jp
     * @brief RTObject_impl::readAll()Ă΂鉼z֐
     *
     * DataPort f[^ǂݏo
     *
     * @return true:,false:s
     * @else
     * @brief It is a virtual method that is called from RTObject_impl::readAll().
     * This method reads out data from DataPort. 
     *
     * @return true:Success,false:Failure
     * @endif
     */
    virtual bool read() = 0;

    /*!
     * @if jp
     * @brief vpeB擾
     *
     * |[g̃vpeB擾B
     *
     * @return vpeB
     * @else
     * @brief Get properties
     *
     * This method gets properties in the port. 
     *
     * @return Properties
     * @endif
     */
    coil::Properties& properties();

    /*!
     * @if jp
     * @brief Connector 擾
     *
     * ݏLĂRlN^擾B
     *
     * @return connector ̃Xg
     *
     * @else
     *
     * @brief Connector list
     *
     * This operation returns connector list
     *
     * @return connector list
     *
     * @endif
     */
    const std::vector<InPortConnector*>& connectors();

    /*!
     * @if jp
     * @brief ConnectorProfile 擾
     *
     * ݏLĂRlN^Profile擾B
     *
     * @return ConnectorProfile ̃Xg
     *
     * @else
     *
     * @brief ConnectorProfile list
     *
     * This operation returns ConnectorProfile list
     *
     * @return connector list
     *
     * @endif
     */
    ConnectorInfoList getConnectorProfiles();

    /*!
     * @if jp
     * @brief ConnectorId 擾
     *
     * ݏLĂRlN^ID擾B
     *
     * @return ConnectorId ̃Xg
     *
     * @else
     *
     * @brief ConnectorId list
     *
     * This operation returns ConnectorId list
     *
     * @return connector list
     *
     * @endif
     */
    coil::vstring getConnectorIds();

    /*!
     * @if jp
     * @brief Connector̖O擾
     *
     * ݏLĂRlN^̖O擾B
     *
     * @return Connector̃Xg
     *
     * @else
     *
     * @brief Connector name list
     *
     * This operation returns Connector name list
     *
     * @return connector name list
     *
     * @endif
     */
    coil::vstring getConnectorNames();

    /*!
     * @if jp
     * @brief ConnectorProfileIDŎ擾
     *
     * ݏLĂRlN^IDŎ擾B
     *
     * @param id Connector ID
     * @return RlN^ւ̃|C^
     *
     * @else
     *
     * @brief Getting ConnectorProfile by ID
     *
     * This operation returns Connector specified by ID.
     *
     * @param id Connector ID
     * @return A pointer to connector
     *
     * @endif
     */
    InPortConnector* getConnectorById(const char* id);

     /*!
     * @if jp
     * @brief ConnectorProfile𖼑OŎ擾
     *
     * ݏLĂRlN^𖼑OŎ擾B
     *
     * @param name Connector name
     * @return RlN^ւ̃|C^
     *
     * @else
     *
     * @brief Getting Connector by name
     *
     * This operation returns Connector specified by name.
     *
     * @param id Connector ID
     * @return A pointer to connector
     *
     * @endif
     */
   InPortConnector* getConnectorByName(const char* name);

    /*!
     * @if jp
     * @brief ConnectorProfileIDŎ擾
     *
     * ݏLĂRlN^IDŎ擾B
     *
     * @param id Connector ID
     * @param prof ConnectorProfile
     * @return false w肵IDȂ
     *
     * @else
     *
     * @brief Getting ConnectorProfile by name
     *
     * This operation returns ConnectorProfile specified by name
     *
     * @param id Connector ID
     * @param prof ConnectorProfile
     * @return false@specified ID does not exist
     *
     * @endif
     */
    bool getConnectorProfileById(const char* id,
                                 ConnectorInfo& prof);

    /*!
     * @if jp
     * @brief ConnectorProfile𖼑OŎ擾
     *
     * ݏLĂRlN^𖼑OŎ擾B
     *
     * @param name Connector name
     * @param prof ConnectorProfile
     * @return false w肵OȂ
     *
     * @else
     *
     * @brief Getting ConnectorProfile by name
     *
     * This operation returns ConnectorProfile specified by name
     *
     * @param id Connector ID
     * @param prof ConnectorProfile
     * @return false specified name does not exist
     *
     * @endif
     */
    bool getConnectorProfileByName(const char* name,
                                   ConnectorInfo& prof);


    /*!
     * @if jp
     *
     * @brief InPort activates 
     *
     * Port ɓo^ĂSẴC^[tF[X activate B
     *
     * @else
     *
     * @brief Activate all Port interfaces
     *
     * This operation activate all interfaces that is registered in the
     * ports.
     *
     * @endif
     */
    virtual void activateInterfaces();

    /*!
     * @if jp
     *
     * @brief SĂ Port ̃C^[tF[X deactivates 
     *
     * Port ɓo^ĂSẴC^[tF[X deactivate B
     *
     * @else
     *
     * @brief Deactivate all Port interfaces
     *
     * This operation deactivate all interfaces that is registered in the
     * ports.
     *
     * @endif
     */
    virtual void deactivateInterfaces();

    /*!
     * @if jp
     * @brief ConnectorDataListener Xiǉ
     *
     * obt@݂܂͓ǂݏoCxgɊ֘Ae탊Xiݒ肷B
     *
     * ݒł郊Xĩ^CvƃR[obNCxg͈ȉ̒ʂ
     *
     * - ON_BUFFER_WRITE:          obt@ݎ
     * - ON_BUFFER_FULL:           obt@t
     * - ON_BUFFER_WRITE_TIMEOUT:  obt@݃^CAEg
     * - ON_BUFFER_OVERWRITE:      obt@㏑
     * - ON_BUFFER_READ:           obt@ǂݏo
     * - ON_SEND:                  InProtւ̑M
     * - ON_RECEIVED:              InProtւ̑M
     * - ON_SEND_ERTIMEOUT:        OutPort^CAEg
     * - ON_SEND_ERERROR:          OutPortG[
     * - ON_RECEIVER_FULL:         InProtobt@t
     * - ON_RECEIVER_TIMEOUT:      InProtobt@^CAEg
     * - ON_RECEIVER_ERROR:        InProtG[
     *
     * Xi ConnectorDataListener pAȉ̃VOj`
     * operator() ĂKvB
     *
     * ConnectorDataListener::
     *         operator()(const ConnectorProfile&, const cdrStream&)
     *
     * ftHgł́Å֐ɗ^XiIuWFNg̏L
     * OutPortɈڂAOutPort̎́A
     * removeConnectorDataListener() ɂ폜ɎIɉ̂B
     * XiIuWFNg̏LĂяoňێꍇ́A3
     *  false w肵AIȉ̂}邱ƂłB
     *
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     * @param autoclean XiIuWFNg̎Îsǂ̃tO
     *
     * @else
     * @brief Adding BufferDataListener type listener
     *
     * This operation adds certain listeners related to buffer writing and
     * reading events.
     * The following listener types are available.
     *
     * - ON_BUFFER_WRITE:          At the time of buffer write
     * - ON_BUFFER_FULL:           At the time of buffer full
     * - ON_BUFFER_WRITE_TIMEOUT:  At the time of buffer write timeout
     * - ON_BUFFER_OVERWRITE:      At the time of buffer overwrite
     * - ON_BUFFER_READ:           At the time of buffer read
     * - ON_SEND:                  At the time of sending to InPort
     * - ON_RECEIVED:              At the time of finishing sending to InPort
     * - ON_SENDER_TIMEOUT:        At the time of timeout of OutPort
     * - ON_SENDER_ERROR:          At the time of error of OutPort
     * - ON_RECEIVER_FULL:         At the time of bufferfull of InPort
     * - ON_RECEIVER_TIMEOUT:      At the time of timeout of InPort
     * - ON_RECEIVER_ERROR:        At the time of error of InPort
     *
     * Listeners should have the following function operator().
     *
     * ConnectorDataListener::
     *         operator()(const ConnectorProfile&, const cdrStream&)
     *
     * The ownership of the given listener object is transferred to
     * this OutPort object in default.  The given listener object will
     * be destroied automatically in the OutPort's dtor or if the
     * listener is deleted by removeConnectorDataListener() function.
     * If you want to keep ownership of the listener object, give
     * "false" value to 3rd argument to inhibit automatic destruction.
     *
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     * @param autoclean A flag for automatic listener destruction
     *
     * @endif
     */
    void addConnectorDataListener(ConnectorDataListenerType listener_type,
                                  ConnectorDataListener* listener,
                                  bool autoclean = true);


    /*!
     * @if jp
     * @brief ConnectorDataListener Xi폜
     *
     * ݒ肵e탊Xi폜B
     * 
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     *
     * @else
     * @brief Removing BufferDataListener type listener
     *
     * This operation removes a specified listener.
     *     
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     *
     * @endif
     */
    void removeConnectorDataListener(ConnectorDataListenerType listener_type,
                                     ConnectorDataListener* listener);
    

    /*!
     * @if jp
     * @brief ConnectorListener Xiǉ
     *
     * obt@݂܂͓ǂݏoCxgɊ֘Ae탊Xiݒ肷B
     *
     * ݒł郊Xĩ^Cv
     *
     * - ON_BUFFER_EMPTY:       obt@̏ꍇ
     * - ON_BUFFER_READTIMEOUT: obt@Ń^CAEgꍇ
     *
     * Xi͈ȉ̃VOj` operator() ĂKvB
     *
     * ConnectorListener::operator()(const ConnectorProfile&)
     *
     * ftHgł́Å֐ɗ^XiIuWFNg̏L
     * OutPortɈڂAOutPort̎́A
     * removeConnectorListener() ɂ폜ɎIɉ̂B
     * XiIuWFNg̏LĂяoňێꍇ́A3
     *  false w肵AIȉ̂}邱ƂłB
     *
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     * @param autoclean XiIuWFNg̎Îsǂ̃tO
     *
     * @else
     * @brief Adding ConnectorListener type listener
     *
     * This operation adds certain listeners related to buffer writing and
     * reading events.
     * The following listener types are available.
     *
     * - ON_BUFFER_EMPTY:       At the time of buffer empty
     * - ON_BUFFER_READTIMEOUT: At the time of buffer read timeout
     *
     * Listeners should have the following function operator().
     *
     * ConnectorListener::operator()(const ConnectorProfile&)
     *  
     * The ownership of the given listener object is transferred to
     * this OutPort object in default.  The given listener object will
     * be destroied automatically in the OutPort's dtor or if the
     * listener is deleted by removeConnectorListener() function.
     * If you want to keep ownership of the listener object, give
     * "false" value to 3rd argument to inhibit automatic destruction.
     *
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     * @param autoclean A flag for automatic listener destruction
     *
     * @endif
     */
    void addConnectorListener(ConnectorListenerType callback_type,
                              ConnectorListener* listener,
                              bool autoclean = true);

    /*!
     * @if jp
     * @brief ConnectorDataListener Xi폜
     *
     * ݒ肵e탊Xi폜B
     * 
     * @param listener_type Xi^Cv
     * @param listener XiIuWFNgւ̃|C^
     *
     * @else
     * @brief Removing BufferDataListener type listener
     *
     * This operation removes a specified listener.
     *     
     * @param listener_type A listener type
     * @param listener A pointer to a listener object
     *
     * @endif
     */
    void removeConnectorListener(ConnectorListenerType callback_type,
                                 ConnectorListener* listener);

    /*!
     * @if jp
     * @brief endian ݒԂ
     *
     * endian ݒboollԂB
     *
     * @return m_littleEndian littlȅꍇtrueAbig̏ꍇfalse ԂB
     *
     * @else
     * @brief
     *
     * return it whether endian setting.
     *
     *@return Return true in the case of "little", false in "big" than it.
     *
     * @endif
     */
    bool isLittleEndian();

    /*!
     * @if jp
     *
     * @brief [CORBA interface] Port ̐ڑs
     *
     * ^ꂽ ConnectoionProfile ̏ɊÂAPortԂ̐ڑm
     * B̊֐͎ɃAvP[VvOc[Ăяo
     * ƂOƂĂB
     * 
     * @param connector_profile ConnectorProfile
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     *
     * @brief [CORBA interface] Connect the Port
     *
     * This operation establishes connection according to the given
     * ConnectionProfile inforamtion. This function is premised on
     * calling from mainly application program or tools.
     *
     * @param connector_profile The ConnectorProfile.
     * @return ReturnCode_t The return code of ReturnCode_t type.
     *
     * @endif
     */
    virtual ReturnCode_t
    connect(ConnectorProfile& connector_profile)
      throw (CORBA::SystemException);

  protected:

    /*!
     * @if jp
     * @brief InterfaceJ
     *
     * InterfaceJB
     *  ConnectorProfile Ɋi[Ă dataflow_type  push ^
     * ̏ꍇ́Aw肳ꂽ interface_type  InPortProvider Ɋւ
     *  ConnectorProfile::properties ɏ݌Ăяoɖ߂B
     *
     *  dataport.dataflow_type
     *
     * @param connector_profile RlN^vt@C
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     * @brief Publish interface information
     *
     * Publish interface information.
     * Assign the Provider information that owned by this port
     * to ConnectorProfile#properties
     *
     * @param connector_profile The connector profile
     *
     * @return The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t
    publishInterfaces(ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     * @brief Interfaceɐڑ
     *
     * InterfaceɐڑB
     * PortLConsumerɓKProviderɊւ 
     * ConnectorProfile#properties 璊oA
     * ConsumerCORBAIuWFNgQƂݒ肷B
     *
     * @param connector_profile RlN^Evt@C
     *
     * @return ReturnCode_t ^̃^[R[h
     *
     * @else
     * @brief Subscribe to the interface
     *
     * Subscribe to interface.
     * Derive Provider information that matches Consumer owned by the Port 
     * from ConnectorProfile#properties and 
     * set the Consumer to the reference of the CORBA object.
     *
     * @param connector_profile The connector profile
     *
     * @return ReturnCode_t The return code of ReturnCode_t type
     *
     * @endif
     */
    virtual ReturnCode_t
    subscribeInterfaces(const ConnectorProfile& connector_profile);
    
    /*!
     * @if jp
     * @brief Interfaceւ̐ڑ
     *
     * Interfaceւ̐ڑB
     * ^ꂽConnectorProfileɊ֘AConsumerɐݒ肳ꂽSĂObject
     * ڑB
     *
     * @param connector_profile RlN^Evt@C
     *
     * @else
     * @brief Disconnect the interface connection
     *
     * Disconnect the interface connection.
     * Release all objects set in Consumer associated with 
     * given ConnectorProfile and unscribe the interface.
     *
     * @param connector_profile The connector profile
     *
     * @endif
     */
    virtual void
    unsubscribeInterfaces(const ConnectorProfile& connector_profile);


    /*!
     * @if jp
     * @brief InPort provider ̏
     * @else
     * @brief InPort provider initialization
     * @endif
     */
    void initProviders();

    /*!
     * @if jp
     * @brief OutPort consumer ̏
     * @else
     * @brief OutPort consumer initialization
     * @endif
     */
    void initConsumers();

    /*!
     * @if jp
     * @brief VACŨGfBA`FbN
     *
     * ^ꂽvpeBɐݒ肳ĂAf[^̃VACŨG
     * fBAw`FbNBGfBAw肪ȂĂ
     * ΁Atrue ԂA littleEndian ɁAݒlgGfBA
     * ł true ArbOGfBAł false ԂB
     *
     * @param prop `FbNvpeB
     * @param littleEndian GfBAitrue:little,false:bigj
     * @return true:"serializer"L[݂Ȃ ܂ ݂ĂēeB
,false:"serializer"L[݂Ă邪e ܂ ݂Ă邪e"little","big" ȊOB
     *
     * @else
     *
     * @brief Checking endian flag of serializer
     *
     * This operation checks endian flag of data serializer that is
     * specified properties. If valid specification is found, this
     * operation returns true and set argument littleEndian. True
     * means little endian, false means big endian.
     *
     * @param prop Properties
     * @param littleEndian Endian Information(true:little,false:big)
     * @return true:"Serializer" key doesn't exist. or  "Serializer" key exists and there is a content.
     *
     *false:There is no content though "Serializer" key exists. or ithe content is not "Little. " though "Serializer" key exists. or The content is not "little" or "big" though "Serializer" key exists.
     *
     * @endif
     */
    bool checkEndian(const coil::Properties& prop, bool& littleEndian);

    /*!
     * @if jp
     * @brief InPort provider ̐
     *
     * InPortProvider 𐶐A ConnectorProfile ɌJB
     * Ɏsꍇ 0 ԂB
     *
     * @else
     * @brief InPort provider creation
     * @endif
     */
    InPortProvider*
    createProvider(ConnectorProfile& cprof, coil::Properties& prop);

    /*!
     * @if jp
     * @brief OutPort consumer ̐
     *
     * OutPortConsumer 𐶐B
     * Ɏsꍇ 0 ԂB
     *
     * @else
     * @brief InPort provider creation
     * @endif
     */
    OutPortConsumer*
    createConsumer(const ConnectorProfile& cprof, coil::Properties& prop);

    /*!
     * @if jp
     * @brief InPortPushConnector ̐
     *
     * Connector 𐶐A m_connectors ɕۑB
     * Ɏsꍇ 0 ԂB
     *
     * @else
     * @brief InPortPushConnector creation
     * @endif
     */
    InPortConnector*
    createConnector(ConnectorProfile& cprof, coil::Properties& prop,
                    InPortProvider* provider);
    /*!
     * @if jp
     * @brief InPortPullConnector ̐
     *
     * Connector 𐶐A m_connectors ɕۑB
     * Ɏsꍇ 0 ԂB
     *
     * @else
     * @brief InPortPullConnector creation
     * @endif
     */
    InPortConnector*
    createConnector(const ConnectorProfile& cprof, coil::Properties& prop,
                    OutPortConsumer* consumer);

  protected:
    /*!
     * @if jp
     * @brief obt@[h
     *
     * true:single buffer mode.
     * false:multi buffer mode.
     *
     * @else
     * @brief Buffer mode
     *
     * true:single buffer mode.
     * false:multi buffer mode.
     *
     * @endif
     */
    bool m_singlebuffer;
    /*!
     * @if jp
     * @brief obt@
     * @else
     * @brief Buffer
     * @endif
     */
    CdrBufferBase* m_thebuffer;
    /*!
     * @if jp
     * @brief vpeB
     * @else
     * @brief Properties
     * @endif
     */
    coil::Properties m_properties;
    /*!
     * @if jp
     * @brief p\provider
     * @else
     * @brief Available providers
     * @endif
     */
    coil::vstring m_providerTypes;
    /*!
     * @if jp
     * @brief p\consumer
     * @else
     * @brief Available consumers
     * @endif
     */
    coil::vstring m_consumerTypes;
    /*!
     * @if jp
     * @brief ڑXg
     * @else
     * @brief Connection list
     * @endif
     */
    ConnectorList m_connectors;
    /*!
     * @if jp
     * @brief ڑGfBA 
     * @else
     * @brief Connected Endian
     * @endif
     */
    bool m_littleEndian;

    /*!
     * @if jp
     * @brief ConnectorDataListener Xi
     * @else
     * @brief ConnectorDataListener listener
     * @endif
     */
    ConnectorListeners m_listeners;
  };
}; // namespace RTC

#endif // RTC_INPORTBASE_H
