// -*- C++ -*-
/*!
 * @file ManagerServant.h
 * @brief RTComponent manager servant implementation class
 * @date $Date: 2007-12-31 03:08:04 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2008-2010
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */

#ifndef RTM_MANAGERSERVANT_H
#define RTM_MANAGERSERVANT_H

#include <iostream>
#include <coil/Mutex.h>
#include <coil/Guard.h>
#include <rtm/idl/ManagerSkel.h>
#include <rtm/Manager.h>
#include <rtm/SystemLogger.h>

/*!
 * @if jp
 * @namespace RTM
 *
 * @brief RT-Middleware
 *
 *
 * @else
 * @namespace RTM
 *
 * @brief RT-Middleware
 *
 *
 * @endif
 */
namespace RTM
{
  /*!
   * @if jp
   * @brief ManagerCORBANX
   *
   * ManagerCORBAT[ogAOR|[lg̐E폜A
   * VXeԂ̎擾ȂǂsB
   *
   * @else
   * @brief Manager CORBA class
   *
   * This class changes Manager to CORBA Servant.
   * Generation/deletion of the component, to get the state of the system, 
   * etc. can be done from the outside. 
   *
   * @endif
   */
  class ManagerServant
    : public virtual POA_RTM::Manager,
      public virtual PortableServer::RefCountServantBase
  {
  public:
    /*!
     * @if jp
     * @brief RXgN^
     *
     * RXgN^
     *
     * @else
     * @brief Constructor
     *
     * Constructor
     *
     * @endif
     */
    // standard constructor
    ManagerServant();

    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * @else
     * 
     * @brief Virtual destructor
     * 
     * @endif
     */
    virtual ~ManagerServant(void);

    /*!
     * @if jp
     * @brief W[[h
     *
     * Y}l[WɎw肳ꂽW[[hAw肳ꂽ
     * ֐ŏsB
     *
     * @param pathname W[ւ̃pX
     * @param initfunc W[̏֐
     * @return ^[R[h
     *
    // standard constructor
     * @else
     * @brief Loading a module
     *
     * This operation loads a specified loadable moduleAand perform
     * initialization with the specified function.
     *
     * @param pathname A path to a loading module.
     * @param initfunc Module initialization function.
     * @return The return code.
     *
     * @endif
     */
    RTC::ReturnCode_t load_module(const char* pathname, const char* initfunc);

    /*!
     * @if jp
     * @brief W[A[h
     *
     * Y}l[WɎw肳ꂽW[A[hB
     *
     * @param pathname W[ւ̃pX
     * @return ^[R[h
     *
     * @else
     * @brief Unloading a module
     *
     * This operation unloads a specified loadable module.
     *
     * @param pathname A path to a loading module.
     * @return The return code.
     *
     * @endif
     */
    RTC::ReturnCode_t unload_module(const char* pathname);

    /*!
     * @if jp
     * @brief [h\ȃW[̃vt@C擾
     *
     * [h\ȃW[̃vt@C擾B
     *
     * @return W[vt@C
     *
     * @else
     * @brief Getting loadable module profiles
     *
     * This operation returns loadable module profiles.
     *
     * @return A module profile list.
     *
     * @endif
     */
    RTM::ModuleProfileList* get_loadable_modules();

    /*!
     * @if jp
     * @brief [hς݂̃W[̃vt@C擾
     *
     * [hς݂̃W[̃vt@C擾B
     *
     * @return W[vt@C
     *
     * @else
     * @brief Getting loaded module profiles
     *
     * This operation returns loaded module profiles.
     *
     * @return A module profile list.
     *
     * @endif
     */
    RTM::ModuleProfileList* get_loaded_modules();

    // component ֘A
    /*!
     * @if jp
     * @brief R|[lgt@Ng̃vt@C擾
     *
     * [hς݂̃W[̂ARTR|[lg̃W[
     * t@Ng̃vt@C̃Xg擾B
     *
     * @return R|[lgt@Ng̃vt@CXg
     *
     * @else
     * @brief Getting component factory profiles
     *
     * This operation returns component factory profiles from loaded
     * RT-Component module factory profiles.
     *
     * @return An RT-Component factory profile list.
     *
     * @endif
     */
    RTM::ModuleProfileList* get_factory_profiles();

    /*!
     * @if jp
     * @brief R|[lg𐶐
     *
     * Ɏw肳ꂽR|[lg𐶐B
     *
     * @return ꂽRTR|[lg
     *
     * @else
     * @brief Creating an RT-Component
     *
     * This operation creates RT-Component according to the string
     * argument.
     *
     * @return A created RT-Component
     *
     * @endif
     */
    RTC::RTObject_ptr create_component(const char* module_name);

    /*!
     * @if jp
     * @brief R|[lg폜
     *
     * Ɏw肳ꂽR|[lg폜B
     *
     * @return ^[R[h
     *
     * @else
     * @brief Deleting an RT-Component
     *
     * This operation delete an RT-Component according to the string
     * argument.
     *
     * @return Return code
     *
     * @endif
     */
    RTC::ReturnCode_t delete_component(const char* instance_name);

    /*!
     * @if jp
     * @brief ÑR|[lg̃Xg擾
     *
     * ݓY}l[WŋÑR|[lg̃XgԂB
     *
     * @return RTR|[lg̃Xg
     *
     * @else
     * @brief Getting RT-Component list running on this manager
     *
     * This operation returns RT-Component list running on this manager.
     *
     * @return A list of RT-Components
     *
     * @endif
     */
    RTC::RTCList* get_components();

    /*!
     * @if jp
     * @brief ÑR|[lgvt@C̃Xg擾
     *
     * ݓY}l[WŋÑR|[lg̃vt@C̃X
     * gԂB
     *
     * @return RTR|[lgvt@C̃Xg
     *
     * @else
     * @brief Getting RT-Component's profile list running on this manager
     *
     * This operation returns RT-Component's profile list running on
     * this manager.
     *
     * @return A list of RT-Components' profiles
     *
     * @endif
     */
    RTC::ComponentProfileList* get_component_profiles();

    // manager {
    /*!
     * @if jp
     * @brief }l[W̃vt@C擾
     *
     * ݓY}l[W̃vt@C擾B
     *
     * @return }l[Wvt@C
     *
     * @else
     * @brief Getting this manager's profile.
     *
     * This operation returns this manager's profile.
     *
     * @return Manager's profile
     *
     * @endif
     */
    RTM::ManagerProfile* get_profile();

    /*!
     * @if jp
     * @brief }l[W̃RtBM[V擾
     *
     * ݓY}l[W̃RtBM[V擾B
     *
     * @return }l[WRtBM[V
     *
     * @else
     * @brief Getting this manager's configuration.
     *
     * This operation returns this manager's configuration.
     *
     * @return Manager's configuration
     *
     * @endif
     */
    RTM::NVList* get_configuration();

    /*!
     * @if jp
     * @brief }l[W̃RtBM[Vݒ肷
     *
     * ݓY}l[W̃RtBM[Vݒ肷B
     *
     * @param name ZbgRtBM[ṼL[
     * @param value ZbgRtBM[V̒l
     * @return ^[R[h
     *
     * @else
     * @brief Setting manager's configuration
     *
     * This operation sets managers configuration.
     *  
     * @param name A configuration key name to be set
     * @param value A configuration value to be set
     * @return Return code
     *
     * @endif
     */
    RTC::ReturnCode_t set_configuration(const char* name, const char* value);

    /*!
     * @if jp
     * @brief }l[W}X^[ǂ
     *
     * ̊֐̓}l[W}X^[ǂԂBTrueȂ΁AY}
     * l[W̓}X^[łAȊO False ԂB
     *
     * @return }X^[}l[Wǂbooll
     *
     * @else
     * @brief Whether this manager is master or not
     *
     * It returns "True" if this manager is a master, and it returns
     * "False" in other cases.
     *  
     * @return A boolean value that means it is master or not.
     *
     * @endif
     */
    bool is_master();

    /*!
     * @if jp
     * @brief }X^[}l[W̎擾
     *
     * ̃}l[WX[u}l[W̏ꍇA}X^[ƂȂĂ}
     * l[W̃XgԂB̃}l[W}X^[̏ꍇÃX
     * gԂB
     *
     * @return }X^[}l[W̃Xg
     *
     * @else
     * @brief Getting master managers
     *
     * This operation returns master manager list if this manager is
     * slave. If this manager is master, an empty sequence would be
     * returned.
     *  
     * @return Master manager list
     *
     * @endif
     */
    RTM::ManagerList* get_master_managers();

    /*!
     * @if jp
     * @brief }X^[}l[W̒ǉ
     *
     * ̃}l[W̃}X^Ƃă}l[WǉB߂lɂ́A
     * Y}l[WŒǉꂽ}X^[}l[Wʂ郆j[N
     * IDԂB̃}l[W}X^̏ꍇAYIDŎw肳ꂽ
     * }X^[}l[WԂBIDŎw肳ꂽ}X^[}l[WȂ
     * ꍇAnilIuWFNgԂB
     *
     * @return }X^[}l[W
     *
     * @else
     * @brief Getting a master manager
     *
     * This operation returns a master manager with specified id. If
     * the manager with the specified id does not exist, nil object
     * reference would be returned.
     *  
     * @return A master manager
     *
     * @endif
     */
    RTC::ReturnCode_t add_master_manager(RTM::Manager_ptr mgr);

    /*!
     * @if jp
     * @brief }X^[}l[W̍폜
     *
     * ̃}l[Wێ}X^̂Aw肳ꂽ̂폜B
     *
     * @param mgr }X^[}l[W
     * @return ReturnCode_t
     *
     * @else
     * @brief Removing a master manager
     *
     * This operation removes a master manager from this manager.
     * 
     * @param mgr A master manager
     * @return ReturnCode_t 
     *
     * @endif
     */
    RTC::ReturnCode_t remove_master_manager(RTM::Manager_ptr mgr);

    /*!
     * @if jp
     * @brief X[u}l[W̎擾
     *
     * ̃}l[WX[u}l[W̏ꍇAX[uƂȂĂ}
     * l[W̃XgԂB̃}l[WX[ȕꍇÃX
     * gԂB
     *
     * @return X[u}l[W̃Xg
     *
     * @else
     * @brief Getting slave managers
     *
     * This operation returns slave manager list if this manager is
     * slave. If this manager is slave, an empty sequence would be
     * returned.
     *  
     * @return Slave manager list
     *
     * @endif
     */
    RTM::ManagerList* get_slave_managers();

    /*!
     * @if jp
     * @brief X[u}l[W̒ǉ
     *
     * ̃}l[W̃}X^Ƃă}l[WǉB
     *
     * @param mgr X[u}l[W
     * @return ReturnCode_t
     *
     * @else
     * @brief Getting a slave manager
     *
     * This operation add a slave manager to this manager.
     *  
     * @param mgr A slave manager
     * @return ReturnCode_t
     *
     * @endif
     */
    RTC::ReturnCode_t add_slave_manager(RTM::Manager_ptr mgr);

    /*!
     * @if jp
     * @brief X[u}l[W̍폜
     *
     * ̃}l[Wێ}X^̂Aw肳ꂽ̂폜B
     *
     * @param mgr X[u}l[W
     * @return ReturnCode_t
     *
     * @else
     * @brief Removing a slave manager
     *
     * This operation removes a slave manager from this manager.
     * 
     * @param mgr A slave manager
     * @return ReturnCode_t 
     *
     * @endif
     */
    RTC::ReturnCode_t remove_slave_manager(RTM::Manager_ptr mgr);



    /*!
     * @if jp
     * @brief vZX̃Rs[𐶐
     * @return ReturnCode_t
     * @else
     * @brief The copy of the process is generated. 
     * @return ReturnCode_t 
     * @endif
     */
    RTC::ReturnCode_t fork();
    /*!
     * @if jp
     * @brief shutdown
     * @return ReturnCode_t
     * @else
     * @brief This method shutdowns RTC. 
     * @return ReturnCode_t 
     * @endif
     */
    RTC::ReturnCode_t shutdown();
    /*!
     * @if jp
     * @brief ċNB
     * @return ReturnCode_t
     * @else
     * @brief This method restarts RTC.  
     * @return ReturnCode_t 
     * @endif
     */
    RTC::ReturnCode_t restart();

    /*!
     * @if jp
     * @brief RTC̃t@X擾B
     * @return RTC̃t@X
     * @else
     * @brief Get the reference of RTC. 
     * @return RTC reference
     * @endif
     */
    CORBA::Object_ptr get_service(const char* name);

    /*!
     * @if jp
     * @brief Manager̃t@X擾B
     * @return Manager̃t@X
     * @else
     * @brief Get the reference of Manager. 
     * @return Manager reference
     * @endif
     */
    RTM::Manager_ptr getObjRef() const;

    /*!
     * @if jp
     * @brief INSManager̐
     * @return 
     * @else :true, s:false
     * @brief Generate INSManager. 
     * @return Successful:true, Failed:false
     * @endif
     */
    bool createINSManager();

    /*!
     * @if jp
     * @brief Manager̃t@XB
     * @return Manager̃t@X
     * @else
     * @brief Find the reference of Manager. 
     * @return Manager reference
     * @endif
     */
    RTM::Manager_ptr findManager(const char* host_port);
  private:
    typedef coil::Guard<coil::Mutex> Guard;
    ::RTC::Logger rtclog;
    ::RTC::Manager& m_mgr;
    ::RTM::Manager_var m_objref;

    ::RTM::ManagerList m_masters;
    ::coil::Mutex m_masterMutex;
    ::RTM::ManagerList m_slaves;
    ::coil::Mutex m_slaveMutex;

    CORBA::Boolean m_isMaster;

    class is_equiv
    {
      RTM::Manager_var m_mgr;
    public:
      is_equiv(RTM::Manager_ptr mgr)
        : m_mgr(RTM::Manager::_duplicate(mgr)) {}
      bool operator()(RTM::Manager_ptr mgr)
      {
        return m_mgr->_is_equivalent(mgr);
      }
    };

  };
}; // namespace RTM
#endif // RTM_MANAGERSERVANT_H
