// -*- C++ -*-
/*!
 * @file NamingManager.h
 * @brief naming Service helper class
 * @date $Date: 2007-12-31 03:08:04 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: NamingManager.h 1856 2010-02-08 14:45:23Z n-ando $
 *
 */

#ifndef RTC_NAMINGMANAGER_H
#define RTC_NAMINGMANAGER_H

#include <rtm/RTC.h>

#include <coil/Task.h>
#include <coil/Mutex.h>
#include <coil/Guard.h>
#include <rtm/CorbaNaming.h>
#include <rtm/RTObject.h>
#include <rtm/SystemLogger.h>
#include <rtm/ManagerServant.h>

namespace RTC
{
  class Manager;
  /*!
   * @if jp
   *
   * @class NamingBase
   * @brief NamingService ǗpۃNX
   *
   * NamingServer ǗpۃC^[tF[XNXB
   * ۊǗNX́Aȉ̏z֐̎񋟂Ȃ΂ȂȂB
   * - bindObject() : w肵IuWFNgNamingServiceւ̃oCh
   * - unbindObject() : w肵IuWFNgNamingServicẽAoCh
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class NamingBase
   * @brief NamingService management abstract class
   *
   * This is the abstract interface class for NamingServer management.
   * Concrete management classes must implement the following pure virtual 
   * functions.
   * - bindObject() : Bind the specified object to NamingService
   * - unbindObject() : Unbind the specified object from NamingService
   *
   * @since 0.4.0
   *
   * @endif
   */
  class NamingBase
  {
    typedef coil::Mutex Mutex;
    typedef coil::Guard<Mutex> Guard;
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * @else
     *
     * @brief Constructor
     *
     * @endif
     */
    NamingBase() {};
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    virtual ~NamingBase(void) {};
    
    /*!
     * @if jp
     *
     * @brief w肵IuWFNgNamingServiceփoCh鏃z֐
     *
     * @param name oCh̖
     * @param rtobj oChΏۃIuWFNg
     *
     * @else
     *
     * @brief Pure virtual function to bind the specified objects 
     *        to the NamingService
     *
     * @param name The name to be bound to the NamingService
     * @param rtobj The target objects to be bound to the NamingSerivce
     *
     * @endif
     */
    virtual void bindObject(const char* name, const RTObject_impl* rtobj) = 0;

    /*!
     * @if jp
     *
     * @brief w肵ManagerServantNamingServiceփoCh鏃z֐
     *
     * @param name oCh̖
     * @param rtobj oChΏManagerServant
     *
     * @else
     *
     * @brief Pure virtual function to bind the specified ManagerServants 
     *        to NamingService
     *
     * @param name The name to be bound to the NamingService
     * @param rtobj The target objects to be bound to the NamingSerivce
     *
     * @endif
     */
    virtual void bindObject(const char* name,
                            const RTM::ManagerServant* mgr) = 0;
    
    /*!
     * @if jp
     *
     * @brief w肵IuWFNgNamingServiceAoCh邽߂
     *        z֐
     *
     * @param name AoChΏۃIuWFNg
     *
     * @else
     *
     * @brief Pure virtual function to unbind the specified objects from 
     *        NamingService
     *
     * @param name The name of the object released from NamingService
     *
     * @endif
     */
    virtual void unbindObject(const char* name) = 0;

    /*!
     * @if jp
     *
     * @brief l[T[o̐mFB
     * 
     * @return true:Ă, false:ĂȂ
     *
     * @else
     *
     * @brief Check if the name service is alive
     * 
     * @return true: alive, false:non not alive
     *
     * @endif
     */
    virtual bool isAlive() = 0;
  };
  

  /*!
   * @if jp
   *
   * @class NamingOnCorba
   * @brief CORBA p NamingServer ǗNX
   *
   * CORBA p NamingServer ǗpNXB
   * CORBA R|[lg NamingService ւ̓o^AȂǂǗB
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class NamingOnCorba
   * @brief NamingServer management class for CORBA
   *
   * NamingServer management class for CORBA.
   * Manage to register and unregister CORBA components to NamingService.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class NamingOnCorba
    : public virtual NamingBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * RXgN^B2ɗ^l[T[rX́Al[T[r
     * X̃zXgƃ|[gԍ ":" ŋ؂̂łB|[gԍ
     * ȗꂽꍇA2809ԃ|[ggpB
     *
     * @param orb ORB
     * @param names NamingServer 
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor. Naming service name that is given at the second
     * argument is host name and port number hoined with ":". If the
     * port number is abbreviated, the default port number 2809 is
     * used.
     *
     * @param orb ORB
     * @param names Name of NamingServer
     *
     * @endif
     */
    NamingOnCorba(CORBA::ORB_ptr orb, const char* names);
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    virtual ~NamingOnCorba(void){};
    
    /*!
     * @if jp
     *
     * @brief w肵 CORBA IuWFNgNamingServiceփoCh
     * 
     * w肵 CORBA IuWFNgw肵̂ CORBA NamingService 
     * oChB
     * 
     * @param name oCh̖
     * @param rtobj oChΏۃIuWFNg
     *
     * @else
     *
     * @brief Bind the specified CORBA objects to NamingService
     * 
     * Bind the specified CORBA objects to CORBA NamingService
     * by specified names.
     * 
     * @param name Names at the binding
     * @param rtobj The target objects for the binding
     *
     * @endif
     */
    virtual void bindObject(const char* name, const RTObject_impl* rtobj);
    /*!
     * @if jp
     *
     * @brief w肵ManagerServantNamingServiceփoCh
     *
     * @param name oCh̖
     * @param rtobj oChΏManagerServant
     *
     * @else
     *
     * @brief Bind the specified ManagerServants to NamingService
     *
     * @param name Names at the binding
     * @param mgr The target ManagerServants for the binding
     *
     * @endif
     */
    virtual void bindObject(const char* name, const RTM::ManagerServant* mgr);
    
    /*!
     * @if jp
     *
     * @brief w肵 CORBA IuWFNgNamingServiceAoCh
     * 
     * w肵 CORBA IuWFNg CORBA NamingService AoChB
     * 
     * @param name AoChΏۃIuWFNg
     *
     * @else
     *
     * @brief Unbind the specified CORBA objects from NamingService
     * 
     * Unbind the specified CORBA objects from CORBA NamingService.
     * 
     * @param name The target objects for the unbinding
     *
     * @endif
     */
    virtual void unbindObject(const char* name);
    
    /*!
     * @if jp
     *
     * @brief l[T[o̐mFB
     * 
     * @return true:Ă, false:ĂȂ
     *
     * @else
     *
     * @brief Check if the name service is alive
     * 
     * @return true: alive, false:non not alive
     *
     * @endif
     */
    virtual bool isAlive();

  private:
    Logger rtclog;
    CorbaNaming m_cosnaming;
    std::string m_endpoint;
    bool m_replaceEndpoint;
    std::map<std::string, RTObject_impl*> m_names;
  };
  
  /*!
   * @if jp
   *
   * @class NamingManager
   * @brief NamingServer ǗNX
   *
   * NamingServer ǗpNXB
   * R|[lgNamingServiceւ̓o^AȂǂǗB
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class NamingManager
   * @brief NamingServer management class
   *
   * NamingServer management class.
   * Manage to register and unregister components to NamingService.
   *
   * @since 0.4.0
   *
   * @endif
   */
  class NamingManager
  {
    typedef coil::Mutex Mutex;
    typedef coil::Guard<Mutex> Guard;
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param manager }l[WIuWFNg
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor
     *
     * @param manager Manager object
     *
     * @endif
     */
    NamingManager(Manager* manager);
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    virtual ~NamingManager(void);
    
    /*!
     * @if jp
     *
     * @brief NameServer ̓o^
     *
     * w肵` NameServer o^B
     * ݎw\Ȍ` CORBA ̂݁B
     *
     * @param method NamingService ̌`
     * @param name_server o^ NameServer ̖
     *
     * @else
     *
     * @brief Regster the NameServer
     *
     * Register NameServer by specified format.
     * Currently. only CORBA can be specified.
     *
     * @param method Format of NamingService
     * @param name_server Name of NameServer for registration
     *
     * @endif
     */
    void registerNameServer(const char* method, const char* name_server);
    
    /*!
     * @if jp
     *
     * @brief w肵IuWFNgNamingServiceփoCh
     * 
     * w肵IuWFNgw肵̂ CORBA NamingService փoC
     * hB
     * 
     * @param name oCh̖
     * @param rtobj oChΏۃIuWFNg
     *
     * @else
     *
     * @brief Bind the specified objects to NamingService
     * 
     * Bind the specified objects to CORBA NamingService by specified names.
     * 
     * @param name Names at the binding
     * @param rtobj The target objects for the binding
     *
     * @endif
     */
    void bindObject(const char* name, const RTObject_impl* rtobj);

    /*!
     * @if jp
     *
     * @brief w肵ManagerServantNamingServiceփoCh
     * 
     * w肵ManagerServantw肵̂ CORBA NamingService փo
     * ChB
     * 
     * @param name oCh̖
     * @param mgr oChΏManagerServant
     *
     * @else
     *
     * @brief Bind the specified ManagerServants to NamingService
     * 
     * Bind the specified ManagerServants to CORBA NamingService 
     * by specified names.
     * 
     * @param name Names at the binding
     * @param mgr The target ManagerServants for the binding
     *
     * @endif
     */
    void bindObject(const char* name, const RTM::ManagerServant* mgr);
    
    /*!
     * @if jp
     *
     * @brief NamingServer ̏̍XV
     * 
     * ݒ肳Ă NameServer ɓo^ĂIuWFNg̏
     * XVB
     * 
     * @else
     *
     * @brief Update information of NamingServer
     * 
     * Update the object information registered in the specified NameServer.
     * 
     * @endif
     */
    void update();
    
    /*!
     * @if jp
     *
     * @brief w肵IuWFNgNamingServiceAoCh
     * 
     * w肵IuWFNg NamingService AoChB
     * 
     * @param name AoChΏۃIuWFNg
     *
     * @else
     *
     * @brief Unbind the specified objects from NamingService
     * 
     * Unbind the specified objects from NamingService.
     * 
     * @param name The target objects for the unbinding
     *
     * @endif
     */
    void unbindObject(const char* name);
    
    /*!
     * @if jp
     *
     * @brief SẴIuWFNgNamingServiceAoCh
     * 
     * SẴIuWFNg CORBA NamingService AoChB
     * 
     * @else
     *
     * @brief Unbind all objects from NamingService
     * 
     * Unbind all objects from CORBA NamingService.
     * 
     * @endif
     */
    void unbindAll();
    
    /*!
     * @if jp
     *
     * @brief oChĂSẴIuWFNg擾
     * 
     * oChĂSẴIuWFNg 擾B
     *
     * @return oChς݃IuWFNg Xg
     * 
     * @else
     *
     * @brief Get all bound objects
     * 
     * Get all bound objects.
     *
     * @return Bound object list
     * 
     * @endif
     */
    std::vector<RTObject_impl*> getObjects();
    
  protected:
    /*!
     * @if jp
     *
     * @brief NameServer ǗpIuWFNg̐
     * 
     * w肵^NameServer ǗpIuWFNg𐶐B
     *
     * @param method NamingService `
     * @param name_server NameServer 
     * 
     * @return  NameServer IuWFNg
     * 
     * @else
     *
     * @brief Create objects for NameServer management
     * 
     * Create objects of specified type for NameServer management.
     *
     * @param method NamingService format
     * @param name_server NameServer name
     * 
     * @return Created NameServer objects
     * 
     * @endif
     */
    NamingBase* createNamingObj(const char* method, const char* name_server);
    
    /*!
     * @if jp
     *
     * @brief ݒς݃R|[lg NameServer ɓo^
     * 
     * ݒς݃R|[lgw肵 NameServer ɓo^B
     *
     * @param ns o^Ώ NameServer
     * 
     * @else
     *
     * @brief Register the configured component to NameServer
     * 
     * Register the already configured components to NameServer.
     *
     * @param ns The target NameServer for the registration
     * 
     * @endif
     */
    void bindCompsTo(NamingBase* ns);
    
    /*!
     * @if jp
     *
     * @brief NameServer ɓo^R|[lg̐ݒ
     * 
     * NameServer ɓo^R|[lgݒ肷B
     *
     * @param name R|[lg̓o^
     * @param rtobj o^ΏۃIuWFNg
     * 
     * @else
     *
     * @brief Configure the components that will be registered to NameServer
     * 
     * Configure the components that will be registered to NameServer.
     *
     * @param name Names of components at the registration
     * @param rtobj The target objects for registration
     * 
     * @endif
     */
    void registerCompName(const char* name, const RTObject_impl* rtobj);

    /*!
     * @if jp
     *
     * @brief NameServer ɓo^ManagerServant̐ݒ
     * 
     * NameServer ɓo^ManagerServantݒ肷B
     *
     * @param name ManagerServant̓o^
     * @param mgr o^ΏManagerServant
     * 
     * @else
     *
     * @brief Configure the ManagerServants that will be registered 
     * to NameServer
     * 
     * Configure the ManagerServants that will be registered to NameServer.
     *
     * @param name Names of ManagerServants at the registration
     * @param mgr The target ManagerServants for registration
     * 
     * @endif
     */
    void registerMgrName(const char* name, const RTM::ManagerServant* mgr);
    
    /*!
     * @if jp
     *
     * @brief NameServer ɓo^R|[lg̐ݒ
     * 
     * NameServer ɓo^R|[lg̐ݒB
     *
     * @param name ݒΏۃR|[lg̖
     * 
     * @else
     *
     * @brief Unregister the components that will be registered to NameServer
     * 
     * Unregister the components that will be registered to NameServer.
     *
     * @param name Names of the target components for unregistration
     * 
     * @endif
     */
    void unregisterCompName(const char* name);

    /*!
     * @if jp
     *
     * @brief NameServer ɓo^ManagerServant̐ݒ
     * 
     * NameServer ɓo^ManagerServant̐ݒB
     *
     * @param name ݒΏManagerServant̖
     * 
     * @else
     *
     * @brief Unregister the ManagerServants that will be registered 
     * to NameServer
     * 
     * Unregister the ManagerServants that will be registered to NameServer.
     *
     * @param name Names of the target ManagerServants for unregistration
     * 
     * @endif
     */
    void unregisterMgrName(const char* name);

    /*!
     * @if jp
     *
     * @brief R|lgoCh
     * 
     * l[T[oƐڑăR|lgoChB
     *
     * @param ns NameServer
     * 
     * @else
     *
     * @brief Rebind the component to NameServer
     * 
     * Connect with the NameServer and rebind the component. 
     *
     * @param ns NameServer
     * 
     * @endif
     */
    class Names;
    void retryConnection(Names* ns);
    
  protected:
    // Name Servers' method/name and object
    /*!
     * @if jp
     * @brief NameServer Ǘp\
     * @else
     * @brief Structure for NameServer management
     * @endif
     */
    class Names
    {
    public:
      Names(const char* meth, const char* name, NamingBase* naming)
	: method(meth), nsname(name), ns(naming)
      {
      }
      
      ~Names()
      {
        delete ns;
      }
      
      std::string method;
      std::string nsname;
      NamingBase* ns;
    };
    /*!
     * @if jp
     * @brief NameServer Xg
     * @else
     * @brief NameServer list
     * @endif
     */
    std::vector<Names*> m_names;
    /*!
     * @if jp
     * @brief NameServer Xgmutex
     * @else
     * @brief Mutex of NameServer list
     * @endif
     */
    Mutex m_namesMutex;
    
    // Components' name and object
    /*!
     * @if jp
     * @brief R|[lgǗp\
     * @else
     * @brief Structure for component management
     * @endif
     */
    struct Comps
    {
      Comps(const char* n, const RTObject_impl* obj)
	: name(n), rtobj(obj)
      {}
      std::string name;
      const RTObject_impl* rtobj;
    };
    /*!
     * @if jp
     * @brief ManagerServantǗp\
     * @else
     * @brief Structure for ManagerServant management
     * @endif
     */
    struct Mgr
    {
      Mgr(const char* n, const RTM::ManagerServant* obj)
	: name(n), mgr(obj)
      {}
      std::string name;
      const RTM::ManagerServant* mgr;
    };
    /*!
     * @if jp
     * @brief R|[lgXg
     * @else
     * @brief Component list
     * @endif
     */
    std::vector<Comps*> m_compNames;
    /*!
     * @if jp
     * @brief R|[lgXgmutex
     * @else
     * @brief Mutex of Component list
     * @endif
     */
    Mutex m_compNamesMutex;
    /*!
     * @if jp
     * @brief ManagerServantXg
     * @else
     * @brief ManagerServant list
     * @endif
     */
    std::vector<Mgr*> m_mgrNames;
    /*!
     * @if jp
     * @brief ManagerServantXgmutex
     * @else
     * @brief Mutex of ManagerServant list
     * @endif
     */
    Mutex m_mgrNamesMutex;
    
    /*!
     * @if jp
     * @brief }l[WIuWFNg
     * @else
     * @brief Manager object
     * @endif
     */
    Manager* m_manager;
    
    /*!
     * @if jp
     * @brief K[Xg[
     * @else
     * @brief Logger stream
     * @endif
     */
    Logger rtclog;
  }; // class NamingManager
}; // namespace RTC

#endif // RTC_NAMINGMANAGER_H
