// -*- C++ -*-
/*!
 * @file OutPort.h
 * @brief OutPort class
 * @date $Date: 2007-12-31 03:08:05 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2009
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: OutPort.h 2259 2012-01-04 13:18:29Z n-ando $
 *
 */

#ifndef RTC_OUTPORT_H
#define RTC_OUTPORT_H

#include <iostream>
#include <string>

#include <coil/TimeValue.h>
#include <coil/Time.h>
#include <coil/TimeMeasure.h>
#include <coil/OS.h>

#include <rtm/RTC.h>
#include <rtm/Typename.h>
#include <rtm/OutPortBase.h>
#include <rtm/CdrBufferBase.h>
#include <rtm/PortCallback.h>
#include <rtm/OutPortConnector.h>

template <class DataType>
void setTimestamp(DataType& data)
{
      // set timestamp
      coil::TimeValue tm(coil::gettimeofday());
      data.tm.sec  = tm.sec();
      data.tm.nsec = tm.usec() * 1000;
}

namespace RTC
{
  /*!
   * @if jp
   *
   * @class OutPort
   *
   * @brief OutPort ev[gNX
   * 
   * OutPort ev[gNXAev[g̈ƂāAOutPort
   * f[^^ DataType ƂB
   *
   *
   *
   * OnWritenR[obN (݂ɋNCxgɂR[)
   *
   * - void OnWrite<DataType>::operator()(const DataType): 
   *     OutPort::write() Ăяo݂sۂɃR[B
   *
   * - DataType OnWriteConvert<DataType>::operator()(const DataType): 
   *     OutPort::write() ĂяoAf[^obt@ɏޑOɌĂ΂
   *     f[^̕ϊsBoperator()()̖߂lVACYobt@
   *     ܂B
   *
   *
   * OnBuffernR[obN (obt@ɋNCxgɂR[)
   *
   * 2ނ̃R[obNt@N^̃VOj`
   * ConnectorDataCallback::operator()(ConnectorId, const cdrStream&)
   * ConnectorCallback::operator()(ConnectorId)
   *
   * - void OnBufferWrite::operator()(ConnectorId, cdrStream):
   *     BufferBase::write() R[ɒPɌĂяoR[obNB
   *     ɂwrite()VACYς݂̃f[^^B        
   *   
   * - void OnBufferFull::operator()(ConnectorId, cdrStream):
   *     BufferBase::write() R[ɁAobt@ς̏ꍇ
   *     ĂяoR[obNBRlN^IDя߂Ȃf[^
   *     ɗ^B
   *
   * - void OnBufferWriteTimeout::operator()(ConnectorId, cdrStream):
   *     ConnectorBase::write() R[ɁAobt@ςŏ݂
   *     ^CAEgꍇɌĂяoR[obNBRlN^ID
   *     я߂Ȃf[^ɗ^B
   *
   * - void OnBufferOverwrite::operator()(ConnectorId, cdrStream):
   *     BufferBase::write() R[ɁAobt@㏑[hɐݒ
   *     ĂAobt@ς̏ꍇɌĂяoR[obNB
   *     RlN^IDɗ^B
   *
   *
   * - void OnBufferRead::operator()(ConnectorId, cdrStream):
   *     BufferBase::read() R[ɒPɌĂяoR[obNB
   *     ɂ read() ŕԂVACYς݂̃f[^^B
   *
   * - void OnBufferEmpty::operator()(ConnectorId): 
   *     RlN^obt@ǂ݂ۂɁAobt@̏ꍇɌĂяoB
   *     RlN^IDɗ^B
   *
   * - void OnBufferReadTimeout::operator()(ConnectorId):
   *     RlN^obt@ǂ݂ۂɁAobt@łAǂݏo
   *     ^CAEgꍇɌĂяoR[obNBRlN^ID
   *     ɗ^B
   *
   *
   * OnConnectnR[obN (ڑɋNCxgɂR[)
   *
   * 1ނ̃R[obNt@N^̃VOj`
   * ConnectionCallback::operator()(ConnectorProfile)
   * 
   * - void OnConnect::operator()(ConnectorProfile):
   *     |[g̐ڑɌĂяoR[obNBConnectorProfile
   *     ^B
   *
   * - void OnDisconnect::operator()(ConnectorId):
   *     |[g̐ڑؒfɌĂяoR[obNBɃRlN^ID
   *     ^B  
   *
   * - void OnConnectionLost::operator()(ConnectorId):
   *     |[g̐ڑXgꍇɌĂяoR[obNB
   *     ɃRlN^ID^BOutPort́A葤InPortƂ
   *     ڑXgꍇAڑIɐؒf̂ŁA
   *      OnDisconnect R[obNĂяoB
   *

   *
   *
   * - void OnSend:operator()(ConnectorId, cdrStream):
   *     f[^InPortɑ΂đۂɌĂяoR[obNB
   *     ɃRlN^ID^B
   *
   * - void OnReceived::operator()(ConnectorId, cdrStream):
   *     f[^̑MюMۂɌĂяoR[obNB
   *     ɂ́ARlN^ID^B
   *

   *
   *
   * OnSendernR[obN (MɋNCxgɂR[)
   *   ȉ́ApubV݂ڑAȂ킿 dataflow type = push 
   *   ̂Ƃ̂ݗLB
   *
   * - void OnSenderTimeout::operator()(ConnectorId, cdrStream):
   *     f[^InPortɑ΂đꂽAM^CAEgۂ
   *     ĂяoR[obNBɃRlN^ID^B
   *     InPort̃voC_уRV[}ΉĂꍇɌLB
   *
   * - void OnSenderError::operator()(ConnectorId, cdrStream):
   *     f[^InPortɑ΂đꂽA炩̃G[InPort
   *     ԂꍇɌĂяoR[obNB
   *     ɂ́ARlN^ID^B
   *
   *
   * OnReceivernR[obN (MɋNCxgɂR[)
   *   ȉ́ApubV݂ڑAȂ킿 dataflow type = push 
   *   ̂Ƃ̂ݗLB
   * 
   * - void OnReceiverFull::operator()(ConnectorId, cdrStream):
   *     f[^InPortɑ̂́AInPort̃obt@ς̏ꍇ
   *     ʒm邽߂ɌĂ΂R[obNB
   *     ɂ́ARlN^ID^B
   *
   * - void OnReceiverTimeout::operator()(ConnectorId, cdrStream):
   *     f[^InPortɑ̂́AInPort̃obt@ς
   *     ^CAEgꍇɂʒm邽߂ɌĂ΂R[obNB
   *     ɂ́ARlN^ID^B
   *
   * - void OnReceiverError::operator()(ConnectorId, cdrStream):
   *     f[^InPortɑ̂́AInPortŉ炩̃G[Ԃ
   *     ꍇɌĂяoR[obNB
   *     ɂ́ARlN^ID^B
   *
   *
   *
   *
   *
   * @since 0.2.0
   *
   * @else
   *
   * @brief OutPort template class
   * 
   * This is the OutPort template class. 
   * The data type "DateType" supported by OutPort and the buffer type
   * "BufferType" are used as template arguments.
   *
   * @since 0.2.0
   *
   * @endif
   */
  template <class DataType>
  class OutPort
    : public OutPortBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief RXgN^
     *
     * RXgN^
     *
     * @param name |[g
     * @param value ̃|[gɃoChf[^ϐ
     * @param length obt@(ftHgl:8)
     *
     * @else
     *
     * @brief Constructor
     *
     * Constructor
     *
     * @param name Port's name
     * @param value Data variable bound to this port
     * @param length Buffer length (The default value:8)
     *
     * @endif
     */
    OutPort(const char* name, DataType& value)
      : OutPortBase(name, toTypename<DataType>()), m_value(value),
	m_onWrite(0), m_onWriteConvert(0)
    {
    }
    
    /*!
     * @if jp
     *
     * @brief fXgN^
     * 
     * fXgN^
     * 
     * @else
     *
     * @brief Destructor
     * 
     * Destructor
     * 
     * @endif
     */
    virtual ~OutPort(void)
    {
    }
    
    /*!
     * @if jp
     *
     * @brief f[^
     *
     * |[gփf[^ށB
     *
     * - R[obNt@N^ OnWrite ZbgĂꍇA
     *   OutPort ێobt@ɏޑO OnWrite Ă΂B
     * - OutPort ێobt@I[o[t[ołobt@łA
     *   AލۂɃobt@I[o[t[oꍇA
     *   R[obNt@N^ OnOverflow Ă΂B
     * - R[obNt@N^ OnWriteConvert ZbgĂꍇA
     *   obt@ݎɁA OnWriteConvert  operator() ̖߂l
     *   obt@ɏ܂B
     *
     * @param value ݑΏۃf[^
     *
     * @return ݏ(ݐ:trueAݎs:false)
     *
     * @else
     *
     * @brief Write data
     *
     * Write data in the port.
     *
     * - When callback functor OnWrite is already set, OnWrite will be
     *   invoked before writing into the buffer held by OutPort.
     * - When the buffer held by OutPort can detect the overflow,
     *   and when it detected the overflow at writing, callback functor
     *   OnOverflow will be invoked.
     * - When callback functor OnWriteConvert is already set, 
     *   the return value of operator() of OnWriteConvert will be written
     *   into the buffer at the writing.
     *
     * @param value The target data for writing
     *
     * @return Writing result (Successful:true, Failed:false)
     *
     * @endif
     */
    virtual bool write(DataType& value)
    {
      RTC_TRACE(("DataType write()"))

      if (m_onWrite != NULL)
        {
          (*m_onWrite)(value);
          RTC_TRACE(("OnWrite called"))
        }

      // check number of connectors
      size_t conn_size(m_connectors.size());
      if (!(conn_size > 0)) { return false; }
        
      bool result(true);
      m_status.resize(conn_size);

      for (size_t i(0), len(conn_size); i < len; ++i)
        {
          ReturnCode ret;
          if (m_onWriteConvert != NULL)
            {
              RTC_DEBUG(("m_connectors.OnWriteConvert called"));
              ret = m_connectors[i]->write(((*m_onWriteConvert)(value)));
            }
          else
            {
              RTC_DEBUG(("m_connectors.write called"));
              ret = m_connectors[i]->write(value);
            }
          m_status[i] = ret;
          if (ret == PORT_OK) { continue; }

          result = false;
          const char* id(m_connectors[i]->profile().id.c_str());
          RTC::ConnectorProfile prof(findConnProfile(id));

          if (ret == CONNECTION_LOST)
            {
              RTC_WARN(("connection_lost id: %s", m_connectors[i]->profile().id.c_str()));
              if (m_onConnectionLost != 0)
                {
                  (*m_onConnectionLost)(prof);
                }
              disconnect(m_connectors[i]->id());
            }
        }
      return result;
    }
    
    /*!
     * @if jp
     *
     * @brief f[^
     *
     * |[gփf[^ށB
     * oChꂽϐɐݒ肳ꂽl|[gɏށB
     *
     * @return ݏ(ݐ:trueAݎs:false)
     *
     * @else
     *
     * @brief Write data
     *
     * Write data to the port.
     * Write the value, which was set to the bound variable, to the port.
     *
     * @return Writing result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool write()
    {
      return write(m_value);
    }
    
    /*!
     * @if jp
     *
     * @brief f[^
     *
     * |[gփf[^ށB
     * ݒ肳ꂽl|[gɏށB
     *
     * @param value ݑΏۃf[^
     *
     * @return ݏ(ݐ:trueAݎs:false)
     *
     * @else
     *
     * @brief Write data
     *
     * Write data to the port.
     * Write the set value to the port.
     *
     * @param value The target data for writing
     *
     * @return Writing result (Successful:true, Failed:false)
     *
     * @endif
     */
    bool operator<<(DataType& value)
    {
      return write(value);
    }

    /*!
     * @if jp
     *
     * @brief ̃RlN^ւ̏݃Xe[^X𓾂
     *
     * OutPort ͐ڑƂ Connector ƌĂ΂鉼zf[^`l
     * Bwrite() ֐͂ Connector ɑ΂ăf[^ނA
     * e Connector ͏݂ƂɃXe[^XԂBwrite() ֐ł́A
     * ׂĂ Connector IƂ̂ true ԂAȊO
     * ł false ԋpB̊֐ write()  false ̏ꍇXe[
     * ^X𒲂ׂ̂Ɏgp邱ƂłB
     * 
     * @param index Connector  index
     * @return Xe[^X
     *
     * @else
     *
     * @brief Getting specified connector's writing status
     *
     * An OutPort has Connectors that are virtual data stream channel
     * for each connection.  "write()" function write into these
     * Connectors, and each Connector returns writing-status.  write()
     * function will return a true value if all Connectors return
     * normal status, and a false value will be returned if at least
     * one Connector failed.  This function can be used to inspect
     * each return status
     *
     * @param index Connector index
     * @return Writing status
     *
     * @endif
     */
    DataPortStatus::Enum getStatus(int index)
    {
      return m_status[index];
    }
    /*!
     * @if jp
     *
     * @brief ̃RlN^ւ̏݃Xe[^XXg𓾂
     *
     * OutPort ͐ڑƂ Connector ƌĂ΂鉼zf[^`l
     * Bwrite() ֐͂ Connector ɑ΂ăf[^ނA
     * e Connector ͏݂ƂɃXe[^XԂBwrite() ֐ł́A
     * ׂĂ Connector IƂ̂ true ԂAȊO
     * ł false ԋpB̊֐ write()  false ̏ꍇXe[
     * ^X𒲂ׂ̂Ɏgp邱ƂłB
     * 
     * @return Xe[^XXg
     *
     * @else
     *
     * @brief Getting specified connector's writing status list
     *
     * An OutPort has Connectors that are virtual data stream channel
     * for each connection.  "write()" function write into these
     * Connectors, and each Connector returns writing-status.  write()
     * function will return a true value if all Connectors return
     * normal status, and a false value will be returned if at least
     * one Connector failed.  This function can be used to inspect
     * each return status
     *
     * @return Writing status list
     *
     * @endif
     */
    DataPortStatusList getStatusList()
    {
      return m_status;
    }

    /*!
     * @if jp
     *
     * @brief OnWrite R[obN̐ݒ
     *
     * write() ĂяoɁAeRlN^ɑ΂ăf[^ޒOɌ
     * ΂ OnWrite R[obNt@N^ݒ肷B̊֐ɂ
     * ݒ肳ꂽt@N^IuWFNg̏ĹAĂяoB
     *  const āÃt@N^IuWFNg̉̐ӔC͌ĂяoɂB
     * t@N^̐ݒꍇɂ͈ 0 nƂŁAR[
     * obNƂȂB
     *
     * @param on_write OnWrite R[obNt@N^
     *
     * @else
     *
     * @brief Set OnWrite callback
     *
     * This operation sets the OnWrite callback functor that is called
     * just before write() operation call.  The ownership of the
     * functor object is owned by caller of this operation.  Therefore
     * caller have to destruct the callback functor object by itself.
     * Giving 0 as an argument for this operation, callback will be
     * disabled.
     *
     * @param on_write OnWrite callback functor
     *
     * @endif
     */
    inline void setOnWrite(OnWrite<DataType>* on_write)
    {
      m_onWrite = on_write;
    }
    
    /*!
     * @if jp
     *
     * @brief OnWriteConvert R[obN̐ݒ
     *
     * write() ĂяoɁAeRlN^ɑ΂ăf[^ݒOɌ
     * ΂Af[^ϊs OnWriteConvert R[obNt@N^
     * 肷Bɒ߂ŏ܂ꂽf[^^A߂lɎۂ
     * RlN^܂̓obt@ɏ܂f[^Ԃt@N^ݒ
     * B̊֐ɂݒ肳ꂽt@N^IuWFNg̏ĹA
     * ĂяoB const āÃt@N^IuWFNg
     * ̉̐ӔC͌ĂяoɂBt@N^̐ݒꍇɂ
     *  0 nƂŁAR[obNƂȂB f[^
     * ɌĂ΂ OnWriteConvert R[obNt@N^ݒ肷B
     * ̃R[obN֐̏ʂ܂B̂ߏ݃f[
     * ^̃tB^O\ƂȂB
     *
     * @param on_wconvert OnWriteConvert R[obNt@N^
     *
     * @else
     *
     * @brief Set OnWriteConvert callback
     *
     * This operation sets the OnWriteConvert callback functor that
     * converts given data and is called just before write() operation
     * call. A recent data is given to argument and return data will
     * be written into connector or its buffer. The ownership of the
     * functor object is owned by caller of this operation.  Therefore
     * caller have to destruct the callback functor object by itself.
     * Giving 0 as an argument for this operation, callback will be
     * disabled.
     *
     * @param on_wconvert OnWriteConvert callback functor
     *
     * @endif
     */
    inline void setOnWriteConvert(OnWriteConvert<DataType>* on_wconvert)
    {
      m_onWriteConvert = on_wconvert;
    }
    
  private:
    std::string m_typename;
    /*!
     * @if jp
     * @brief oCh T ^̕ϐւ̎Q
     * @else
     * @brief The reference to type-T variable that is bound.
     * @endif
     */
    DataType& m_value;
    
    /*!
     * @if jp
     * @brief OnWrite R[obNt@N^ւ̃|C^
     * @else
     * @brief Pointer to OnWrite callback functor
     * @endif
     */
    OnWrite<DataType>* m_onWrite;
    
    /*!
     * @if jp
     * @brief OnWriteConvert R[obNt@N^ւ̃|C^
     * @else
     * @brief Pointer to OnWriteConvert callback functor
     * @endif
     */
    OnWriteConvert<DataType>* m_onWriteConvert;

    coil::TimeMeasure m_cdrtime;

    DataPortStatusList m_status;
  };
}; // namespace RTC

#endif // RTC_OUTPORT_H
