// -*- C++ -*-
/*!
 * @file PublisherBase.h
 * @brief Publisher base class
 * @date $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2010
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PublisherBase.h 1853 2010-02-08 07:00:15Z n-ando $
 *
 */

#ifndef RTC_PUBLISHERBASE_H
#define RTC_PUBLISHERBASE_H

#include <coil/Properties.h>
#include <coil/Factory.h>
#include <rtm/RTC.h>
#include <rtm/CdrBufferBase.h>
#include <rtm/DataPortStatus.h>

namespace coil
{
  class Properties;
}
namespace RTC
{
  class InPortConsumer;
  class ConnectorListeners;
  class ConnectorInfo;

  /*!
   * @if jp
   *
   * @class PublisherBase
   *
   * @brief Publisher NX
   * 
   * f[^o^C~OǗđo쓮Publisher* ̊NXB
   * e Publisher ͂̃NXpďڍׂB
   *
   * @since 0.4.0
   *
   * @else
   *
   * @class PublisherBase
   *
   * @brief Base class of Publisher.
   *
   * This is a base class of Publisher*. This class manages data send timing.
   * Variation of Publisher* which implements details of Publisher inherits
   * this PublisherBase class.
   *
   * @endif
   */
  class PublisherBase
    : public DataPortStatus
  {
  public:
    DATAPORTSTATUS_ENUM
    /*!
     * @if jp
     *
     * @brief fXgN^
     *
     * @else
     *
     * @brief Destructor
     *
     * @endif
     */
    virtual ~PublisherBase(void){};

    /*!
     * @if jp
     * @brief ݒ菉
     *
     * InPortConsumer̊eݒsBNXł́A^ꂽ
     * PropertiesKvȏ擾ĊeݒsB init() 
     * ́AOutPortProviderエсAڑɂꂼĂ΂
     * \BāÅ֐͕Ă΂邱Ƃz肵ċL
     * qׂłB
     * 
     * @param prop ݒ
     *
     * @else
     *
     * @brief Initializing configuration
     *
     * This operation would be called to configure in initialization.
     * In the concrete class, configuration should be performed
     * getting appropriate information from the given Properties data.
     * This function might be called right after instantiation and
     * connection sequence respectivly.  Therefore, this function
     * should be implemented assuming multiple call.
     *
     * @param prop Configuration information
     *
     * @endif
     */
    virtual ReturnCode init(coil::Properties& prop) = 0;

    /*!
     * @if jp
     * @brief InPortRV[}̃Zbg
     *
     * ̊֐ł́A Publisher Ɋ֘AtRV[}ZbgB
     * RV[}IuWFNgk|C^̏ꍇAINVALID_ARGSԂB
     * ȊȌꍇ́APORT_OK ԂB
     *
     * @param consumer Consumer ւ̃|C^
     * @return ReturnCode PORT_OK I
     *                    INVALID_ARGS ɕsȒl܂܂Ă
     *
     * @else
     * @brief Store InPort consumer
     *
     * This operation sets a consumer that is associated with this
     * object. If the consumer object is NULL, INVALID_ARGS will be
     * returned.
     *
     * @param consumer A pointer to a consumer object.
     * @return ReturnCode PORT_OK normal return
     *                    INVALID_ARGS given argument has invalid value
     *
     * @endif
     */
    virtual ReturnCode setConsumer(InPortConsumer* consumer) = 0;

    /*!
     * @if jp
     * @brief obt@̃Zbg
     *
     * ̊֐ł́A Publisher Ɋ֘Atobt@ZbgB
     * obt@IuWFNgk|C^̏ꍇAINVALID_ARGSԂB
     * ȊȌꍇ́APORT_OK ԂB
     *
     * @param buffer CDR buffer ւ̃|C^
     * @return ReturnCode PORT_OK I
     *                    INVALID_ARGS ɕsȒl܂܂Ă
     *
     * @else
     * @brief Setting buffer pointer
     *
     * This operation sets a buffer that is associated with this
     * object. If the buffer object is NULL, INVALID_ARGS will be
     * returned.
     *
     * @param buffer A pointer to a CDR buffer object.
     * @return ReturnCode PORT_OK normal return
     *                    INVALID_ARGS given argument has invalid value
     *
     * @endif
     */
    virtual ReturnCode setBuffer(BufferBase<cdrMemoryStream>* buffer) = 0;

    /*!
     * @if jp
     * @brief Xiݒ肷B
     *
     * Publisher ɑ΂ăXiIuWFNg ConnectorListeners ݒ肷B
     * e탊XiIuWFNg܂ ConnectorListeners Zbg邱ƂŁA
     * obt@̓ǂݏAf[^̑Mɂ̃XiR[B
     * ConnectorListeners IuWFNg̏L̓|[g܂ RTObject 
     * Publisher 폜 ConnectorListeners ͍폜邱Ƃ͂ȂB
     * ConnectorListeners k|C^̏ꍇ INVALID_ARGS ԂB
     *
     * @param info ConnectorProfile [JIuWFNg ConnectorInfo
     * @param listeners Xi𑽐ێ ConnectorListeners IuWFNg
     * @return PORT_OK      I
     *         INVALID_ARGS sȈ
     * @else
     * @brief Set the listener. 
     *
     * This function sets ConnectorListeners listener object to the
     * Publisher. By setting ConnectorListeners containing various
     * listeners objects, these listeners are called at the time of
     * reading and writing of a buffer, and transmission of data
     * etc. Since the ownership of the ConnectorListeners object is
     * owned by Port or RTObject, the Publisher never deletes the
     * ConnectorListeners object. If the given ConnectorListeners'
     * pointer is NULL, this function returns INVALID_ARGS.
     *
     * @param info ConnectorInfo that is localized object of ConnectorProfile
     * @param listeners ConnectorListeners that holds various listeners
     * @return PORT_OK      Normal return
     *         INVALID_ARGS Invalid arguments
     * @endif
     */
    virtual ReturnCode setListener(ConnectorInfo& info,
                                   ConnectorListeners* listeners) = 0;

    /*!
     * @if jp
     * @brief f[^
     *
     * Publisher ɑ΂ăf[^ށBRV[}AXiK
     * ɐݒ肳ĂȂAPublisher IuWFNg
     * ȂꍇÅ֐ĂяoƃG[R[h
     * PRECONDITION_NOT_MET ԂAݓ͈̑؍sȂB
     *
     * ݂ɑ΂āARV[}tԁARV[}̃G[A
     * RV[}ւ݂̏^CAEgꍇɂ͂ꂼAG[
     * R[h SEND_FULL, SEND_ERROR, SEND_TIMEOUT ԂB
     *
     * ȊÕG[̏ꍇAPORT_ERROR ԂB
     * 
     *
     * @param data ރf[^ 
     * @param sec ^CAEg
     * @param nsec ^CAEg
     *
     * @return PORT_OK             I
     *         PRECONDITION_NO_MET consumer, buffer, listenerK؂ɐݒ
     *                             ĂȂÃIuWFNg̎O
     *                             𖞂ȂꍇB
     *         SEND_FULL           M悪t
     *         SEND_TIMEOUT        M悪^CAEg
     *         CONNECTION_LOST     ڑؒfꂽƂmB
     *
     * @else
     * @brief Write data 
     *
     * This function writes data into the Publisher. If this function
     * is called without initializing correctly such as a consumer,
     * listeners, etc., error code PRECONDITION_NOT_MET will be
     * returned and no operation of the writing etc. will be
     * performed.
     *
     * When publisher writes data, if the consumer returns
     * full-status, returns error, is returned with timeout, error
     * codes BUFFER_FULL, BUFFER_ERROR and BUFFER_TIMEOUT will be
     * returned respectively.
     *
     * In other cases, PROT_ERROR will be returned.
     *
     * @param data Data to be wrote to the buffer
     * @param sec Timeout time in unit seconds
     * @param nsec Timeout time in unit nano-seconds
     * @return PORT_OK             Normal return
     *         PRECONDITION_NO_MET Precondition does not met. A consumer,
     *                             a buffer, listenes are not set properly.
     *         SEND_FULL           Data was sent but full-status returned
     *         SEND_TIMEOUT        Data was sent but timeout occurred
     *         CONNECTION_LOST     detected that the connection has been lost
     *
     * @endif
     */
    virtual ReturnCode write(const cdrMemoryStream& data,
                             unsigned long sec,
                             unsigned long usec) = 0;

    /*!
     * @if jp
     *
     * @brief ANeBumF
     * 
     * Publisher ̓f[^|[gƓ activate/deactivate B
     * activate() / deactivate() ֐ɂāAANeBuԂƔANeB
     * uԂ؂ւB̊֐ɂA݃ANeBuԂAAN
     * eBuԂmF邱ƂłB
     *
     * @return ԊmF(ANeBu:trueAANeBu:false)
     *
     * @else
     *
     * @brief If publisher is active state
     * 
     * A Publisher can be activated/deactivated synchronized with the
     * data port.  The active state and the non-active state are made
     * transition by the "activate()" and the "deactivate()" functions
     * respectively. This function confirms if the publisher is in
     * active state.
     *
     * @return Result of state confirmation
     *         (Active state:true, Inactive state:false)
     *
     * @endif
     */
    virtual bool isActive() = 0;

    /*!
     * @if jp
     * @brief ANeBu
     *
     * Publisher ANeBuB̊֐ĂяoƂɂA
     * PublisherAf[^𑗐MXbhJnB
     * sĂȂȂǂɂAO𖞂ȂꍇAG[R[
     * h PRECONDITION_NOT_MET ԂB
     *
     * @return PORT_OK I
     *         PRECONDITION_NOT_MET O𖞂Ȃ
     *
     * @else
     * @brief activation
     *
     * This function activates the publisher. By calling this
     * function, this publisher starts the thread that pushes data to
     * InPort. If precondition such as initialization process and so
     * on is not met, the error code PRECONDITION_NOT_MET is returned.
     *
     * @return PORT_OK normal return
     *         PRECONDITION_NOT_MET precondition is not met
     *
     * @endif
     */
    virtual ReturnCode activate() = 0;

    /*!
     * @if jp
     * @brief ANeBu
     *
     * Publisher ANeBuB̊֐ĂяoƂɂA
     * PublisherAf[^𑗐MXbh~B
     * sĂȂȂǂɂAO𖞂ȂꍇAG[R[
     * h PRECONDITION_NOT_MET ԂB
     *
     * @return PORT_OK I
     *         PRECONDITION_NOT_MET O𖞂Ȃ
     *
     * @else
     * @brief deactivation
     *
     * This function deactivates the publisher. By calling this
     * function, this publisher stops the thread that pushes data to
     * InPort. If precondition such as initialization process and so
     * on is not met, the error code PRECONDITION_NOT_MET is returned.
     *
     * @return PORT_OK normal return
     *         PRECONDITION_NOT_MET precondition is not met
     *
     * @endif
     */
    virtual ReturnCode deactivate() = 0;
  
    /*!
     * @if jp
     *
     * @brief Publisher jB
     *
     * Y Publisher jB
     * Y Publisher svɂȂꍇ PublisherFactory ĂяoB
     *
     * @else
     *
     * @brief Release the Publisher
     *
     * Release this Publisher.
     * When Publisher becomes unnecessary, this is invoked from
     * PublisherFactory. 
     *
     * @endif
     */
    virtual void release(){}
  };

  typedef coil::GlobalFactory<PublisherBase> PublisherFactory;

};
#endif // RTC_PUBLISHERBASE_H
