// -*- C++ -*-
/*!
 * @file  PublisherPeriodic.h
 * @brief PublisherPeriodic class
 * @date  $Date: 2007-12-31 03:08:06 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006-2010
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PublisherPeriodic.h 1857 2010-02-08 14:47:22Z n-ando $
 *
 */

#ifndef RTC_PUBLISHERPERIODIC_H
#define RTC_PUBLISHERPERIODIC_H

#include <coil/Task.h>
#include <coil/Mutex.h>
#include <coil/Condition.h>
#include <coil/PeriodicTask.h>

#include <rtm/RTC.h>
#include <rtm/PublisherBase.h>
#include <rtm/CdrBufferBase.h>
#include <rtm/SystemLogger.h>
#include <rtm/ConnectorBase.h>
#include <rtm/ConnectorListener.h>

namespace coil
{
  class Properties;
};

namespace RTC
{
  class InPortConsumer;
  /*!
   * @if jp
   * @class PublisherPeriodic
   * @brief PublisherPeriodic NX
   *
   * IɃf[^𑗐M邽߂ Publisher NXB̃NX́A
   *  Connector ɂāAobt@уRV[}Ɋ֘At
   * BƂɃobt@f[^oRV[}ɑ΂
   * f[^𑗏oB
   *
   * @else
   * @class PublisherPeriodic
   * @brief PublisherPeriodic class
   *
   * Publisher for periodic data transmitting. Usually this class
   * object exists in a Connector object, and it is associated with a
   * buffer and a consumer. This publisher periodically gets data from
   * the buffer and publish it into the consumer.
   *
   * @endif
   */
  class PublisherPeriodic
    : public PublisherBase
  {
  public: 
    typedef coil::Mutex Mutex;
    typedef coil::Condition<Mutex> Condition;
    typedef coil::Guard<coil::Mutex> Guard;
    DATAPORTSTATUS_ENUM

    /*!
     * @if jp
     * @brief RXgN^
     *
     * @else
     * @brief Constructor
     *
     * @endif
     */
    PublisherPeriodic(void);
    
    /*!
     * @if jp
     * @brief fXgN^
     *
     * fXgN^
     *
     * @else
     * @brief Destructor
     *
     * Destructor
     *
     * @endif
     */
    virtual ~PublisherPeriodic(void);
    
    /*!
     * @if jp
     * @brief 
     *
     * ̃NX̃IuWFNggp̂ɐ旧AK̊֐Ă
     * oKvBɂ́ÃIuWFNg̊eݒ܂
     * Properties ^BȂƂAǒĂяoP
     * Hz ̐lƂ PropertyIuWFNg publisher.push_rate L[
     * Ƃvfɐݒ肷KvB 5ms Ȃ킿A200Hz̏ꍇA
     * 200.0 ݒ肷B dataport.publisher.push_rate ݒ̏ꍇA
     * false ԂBf[^vbVۂ̃|V[Ƃ
     * publisher.push_policy L[ƂlɁAall, fifo, skip, new 
     * ꂩ^邱ƂłB
     * 
     * ȉ̃IvV^邱ƂłB
     * 
     * - publisher.thread_type: Xbh̃^Cv (AftHg: default)
     * - publisher.push_rate: PublisheȓM (l)
     * - publisher.push_policy: Push|V[ (all, fifo, skip, new)
     * - publisher.skip_count: L|V skip ̂Ƃ̃XLbv
     * - measurement.exec_time: ^XNsԌv (enable/disable)
     * - measurement.exec_count: ^XN֐sԌv (l, )
     * - measurement.period_time: ^XNԌv (enable/disable)
     * - measurement.period_count: ^XNԌv (l, )
     *
     * @param property {Publisher̋쓮ݒ肵PropertyIuWFNg
     * @return ReturnCode PORT_OK I
     *                    INVALID_ARGS Properties sȒl܂
     *
     * @else
     * @brief Initialization
     *
     * This function have to be called before using this class object.
     * Properties object that includes certain configuration
     * information should be given as an argument.  At least, a
     * numerical value of unit of Hz with the key of
     * "dataport.publisher.push_rate" has to be set to the Properties
     * object of argument.  The value is the invocation cycle of data
     * sending process.  In case of 5 ms period or 200 Hz, the value
     * should be set as 200.0. False will be returned, if there is no
     * value with the key of "dataport.publisher.push_rate".
     *
     * The following options are available.
     * 
     * - publisher.thread_type: Thread type (string, default: default)
     * - publisher.push_rate: Publisher sending period (numberical)
     * - publisher.push_policy: Push policy (all, fifo, skip, new)
     * - publisher.skip_count: The number of skip count in the "skip" policy
     * - measurement.exec_time: Task execution time measurement (enable/disable)
     * - measurement.exec_count: Task execution time measurement count
     *                           (numerical, number of times)
     * - measurement.period_time: Task period time measurement (enable/disable)
     * - measurement.period_count: Task period time measurement count 
     *                             (number, count)
     *
     * @param property Property objects that includes the control information
     *                 of this Publisher
     * @return ReturnCode PORT_OK normal return
     *                    INVALID_ARGS Properties with invalid values.
     * @endif
     */
    virtual ReturnCode init(coil::Properties& prop);

    /*!
     * @if jp
     * @brief InPortRV[}̃Zbg
     *
     * ̊֐ł́A Publisher Ɋ֘AtRV[}ZbgB
     * RV[}IuWFNgk|C^̏ꍇAINVALID_ARGSԂB
     * ȊȌꍇ́APORT_OK ԂB
     *
     * @param consumer Consumer ւ̃|C^
     * @return ReturnCode PORT_OK I
     *                    INVALID_ARGS ɕsȒl܂܂Ă
     *
     * @else
     * @brief Store InPort consumer
     *
     * This operation sets a consumer that is associated with this
     * object. If the consumer object is NULL, INVALID_ARGS will be
     * returned.
     *
     * @param consumer A pointer to a consumer object.
     * @return ReturnCode PORT_OK normal return
     *                    INVALID_ARGS given argument has invalid value
     *
     * @endif
     */
    virtual ReturnCode setConsumer(InPortConsumer* consumer);

    /*!
     * @if jp
     * @brief obt@̃Zbg
     *
     * ̊֐ł́A Publisher Ɋ֘Atobt@ZbgB
     * obt@IuWFNgk|C^̏ꍇAINVALID_ARGSԂB
     * ȊȌꍇ́APORT_OK ԂB
     *
     * @param buffer CDR buffer ւ̃|C^
     * @return ReturnCode PORT_OK I
     *                    INVALID_ARGS ɕsȒl܂܂Ă
     *
     * @else
     * @brief Setting buffer pointer
     *
     * This operation sets a buffer that is associated with this
     * object. If the buffer object is NULL, INVALID_ARGS will be
     * returned.
     *
     * @param buffer A pointer to a CDR buffer object.
     * @return ReturnCode PORT_OK normal return
     *                    INVALID_ARGS given argument has invalid value
     *
     * @endif
     */
    virtual ReturnCode setBuffer(CdrBufferBase* buffer);

    /*!
     * @if jp
     * @brief Xiݒ肷B
     *
     * Publisher ɑ΂ăXiIuWFNg ConnectorListeners ݒ肷B
     * e탊XiIuWFNg܂ ConnectorListeners Zbg邱ƂŁA
     * obt@̓ǂݏAf[^̑Mɂ̃XiR[B
     * ConnectorListeners IuWFNg̏L̓|[g܂ RTObject 
     * Publisher 폜 ConnectorListeners ͍폜邱Ƃ͂ȂB
     * ConnectorListeners k|C^̏ꍇ INVALID_ARGS ԂB
     *
     * @param info ConnectorProfile [JIuWFNg ConnectorInfo
     * @param listeners Xi𑽐ێ ConnectorListeners IuWFNg
     * @return PORT_OK      I
     *         INVALID_ARGS sȈ
     * @else
     * @brief Set the listener. 
     *
     * This function sets ConnectorListeners listener object to the
     * Publisher. By setting ConnectorListeners containing various
     * listeners objects, these listeners are called at the time of
     * reading and writing of a buffer, and transmission of data
     * etc. Since the ownership of the ConnectorListeners object is
     * owned by Port or RTObject, the Publisher never deletes the
     * ConnectorListeners object. If the given ConnectorListeners'
     * pointer is NULL, this function returns INVALID_ARGS.
     *
     * @param info ConnectorInfo that is localized object of ConnectorProfile
     * @param listeners ConnectorListeners that holds various listeners
     * @return PORT_OK      Normal return
     *         INVALID_ARGS Invalid arguments
     * @endif
     */
    virtual ReturnCode setListener(ConnectorInfo& info,
                                   ConnectorListeners* listeners);
    /*!
     * @if jp
     * @brief f[^
     *
     * Publisher ێobt@ɑ΂ăf[^ށBRV[
     * }Aobt@AXiK؂ɐݒ肳ĂȂAPublisher Iu
     * WFNgĂȂꍇÅ֐ĂяoƃG[
     * R[h PRECONDITION_NOT_MET ԂAobt@ւ̏ݓ̑
     * ͈؍sȂB
     *
     * obt@ւ݂̏ƁAInPortւ̃f[^̑M͔񓯊Iɍs
     * 邽߁Å֐́AInPortւ̃f[^M̌ʂA
     * CONNECTION_LOST, BUFFER_FULL Ȃǂ̃^[R[hԂƂ
     * B̏ꍇAf[^̃obt@ւ݂͍̏sȂB
     *
     * obt@ւ݂̏ɑ΂āAobt@tԁAobt@̃G
     * [Aobt@ւ݂̏^CAEgꍇAobt@̎O
     * Ȃꍇɂ͂ꂼAG[R[h BUFFER_FULL,
     * BUFFER_ERROR, BUFFER_TIMEOUT, PRECONDITION_NOT_MET ԂB
     *
     * ȊÕG[̏ꍇAPORT_ERROR ԂB
     * 
     *
     * @param data ރf[^ 
     * @param sec ^CAEg
     * @param nsec ^CAEg
     *
     * @return PORT_OK             I
     *         PRECONDITION_NO_MET consumer, buffer, listenerK؂ɐݒ
     *                             ĂȂÃIuWFNg̎O
     *                             𖞂ȂꍇB
     *         CONNECTION_LOST     ڑؒfꂽƂmB
     *         BUFFER_FULL         obt@tԂłB
     *         BUFFER_ERROR        obt@ɉ炩̃G[ꍇB
     *         NOT_SUPPORTED       T|[gȂ삪sꂽB
     *         TIMEOUT             ^CAEgB
     *
     * @else
     * @brief Write data 
     *
     * This function writes data into the buffer associated with this
     * Publisher.  If a Publisher object calls this function, without
     * initializing correctly such as a consumer, a buffer, listeners,
     * etc., error code PRECONDITION_NOT_MET will be returned and no
     * operation of the writing to a buffer etc. will be performed.
     *
     * Since writing into the buffer and sending data to InPort are
     * performed asynchronously, occasionally this function returns
     * return-codes such as CONNECTION_LOST and BUFFER_FULL that
     * indicate the result of sending data to InPort. In this case,
     * writing data into buffer will not be performed.
     *
     * When publisher writes data to the buffer, if the buffer is
     * filled, returns error, is returned with timeout and returns
     * precondition error, error codes BUFFER_FULL, BUFFER_ERROR,
     * BUFFER_TIMEOUT and PRECONDITION_NOT_MET will be returned
     * respectively.
     *
     * In other cases, PROT_ERROR will be returned.
     *
     * @param data Data to be wrote to the buffer
     * @param sec Timeout time in unit seconds
     * @param nsec Timeout time in unit nano-seconds
     * @return PORT_OK             Normal return
     *         PRECONDITION_NO_MET Precondition does not met. A consumer,
     *                             a buffer, listenes are not set properly.
     *         CONNECTION_LOST     detected that the connection has been lost
     *         BUFFER_FULL         The buffer is full status.
     *         BUFFER_ERROR        Some kind of error occurred in the buffer.
     *         NOT_SUPPORTED       Some kind of operation that is not supported
     *                             has been performed.
     *         TIMEOUT             Timeout occurred when writing to the buffer.
     *
     * @endif
     */
    virtual ReturnCode write(const cdrMemoryStream& data,
                             unsigned long sec,
                             unsigned long usec);
    /*!
     * @if jp
     *
     * @brief ANeBumF
     * 
     * Publisher ̓f[^|[gƓ activate/deactivate B
     * activate() / deactivate() ֐ɂāAANeBuԂƔANeB
     * uԂ؂ւB̊֐ɂA݃ANeBuԂAAN
     * eBuԂmF邱ƂłB
     *
     * @return ԊmF(ANeBu:trueAANeBu:false)
     *
     * @else
     *
     * @brief If publisher is active state
     * 
     * A Publisher can be activated/deactivated synchronized with the
     * data port.  The active state and the non-active state are made
     * transition by the "activate()" and the "deactivate()" functions
     * respectively. This function confirms if the publisher is in
     * active state.
     *
     * @return Result of state confirmation
     *         (Active state:true, Inactive state:false)
     *
     * @endif
     */
    virtual bool isActive();

    /*!
     * @if jp
     * @brief ANeBu
     *
     * Publisher ANeBuB̊֐ĂяoƂɂA
     * PublisherAf[^𑗐MXbhJnB
     * sĂȂȂǂɂAO𖞂ȂꍇAG[R[
     * h PRECONDITION_NOT_MET ԂB
     *
     * @return PORT_OK I
     *         PRECONDITION_NOT_MET O𖞂Ȃ
     *
     * @else
     * @brief activation
     *
     * This function activates the publisher. By calling this
     * function, this publisher starts the thread that pushes data to
     * InPort. If precondition such as initialization process and so
     * on is not met, the error code PRECONDITION_NOT_MET is returned.
     *
     * @return PORT_OK normal return
     *         PRECONDITION_NOT_MET precondition is not met
     *
     * @endif
     */
    virtual ReturnCode activate();

    /*!
     * @if jp
     * @brief ANeBu
     *
     * Publisher ANeBuB̊֐ĂяoƂɂA
     * PublisherAf[^𑗐MXbh~B
     * sĂȂȂǂɂAO𖞂ȂꍇAG[R[
     * h PRECONDITION_NOT_MET ԂB
     *
     * @return PORT_OK I
     *         PRECONDITION_NOT_MET O𖞂Ȃ
     *
     * @else
     * @brief deactivation
     *
     * This function deactivates the publisher. By calling this
     * function, this publisher stops the thread that pushes data to
     * InPort. If precondition such as initialization process and so
     * on is not met, the error code PRECONDITION_NOT_MET is returned.
     *
     * @return PORT_OK normal return
     *         PRECONDITION_NOT_MET precondition is not met
     *
     * @endif
     */
    virtual ReturnCode deactivate();
    
    /*!
     * @if jp
     * @brief Xbhs֐
     *
     * coil::PeriodicTask ɂs^XNs֐B
     *
     * @else
     * @brief Thread execution function
     *
     * A task execution function to be executed by coil::PeriodicTask.
     *
     * @endif
     */
    virtual int svc(void);
    
  protected:
    enum Policy
      {
        ALL,
        FIFO,
        SKIP,
        NEW
      };

    /*!
     * @if jp
     * @brief PushPolicy ̐ݒ
     * @else
     * @brief Setting PushPolicy
     * @endif
     */
    void setPushPolicy(const coil::Properties& prop);

    /*!
     * @if jp
     * @brief Task ̐ݒ
     * @else
     * @brief Setting Task
     * @endif
     */
    bool createTask(const coil::Properties& prop);

    /*!
     * @brief push "all" policy
     */
    ReturnCode pushAll();

    /*!
     * @brief push "fifo" policy
     */
    ReturnCode pushFifo();

    /*!
     * @brief push "skip" policy
     */
    ReturnCode pushSkip();

    /*!
     * @brief push "new" policy
     */
    ReturnCode pushNew();

    /*!
     * @if jp
     * @brief BufferStatus  DataPortStatus ւ̕ϊ
     *
     * obt@̖߂l DataPortStatus::Enum ^֕ϊ֐B
     * ꂼAȉ̂悤ɕϊBϊɃR[obNĂԏꍇA
     * R[oN֐tLB
     * 
     * - BUFFER_OK: PORT_OK
     *  - None
     * - BUFFER_ERROR: BUFFER_ERROR
     *  - None
     * - BUFFER_FULL: BUFFER_FULL
     *  - onBufferFull()
     * - NOT_SUPPORTED: PORT_ERROR
     *  - None
     * - TIMEOUT: BUFFER_TIMEOUT
     *  - onBufferWriteTimeout()
     * - PRECONDITION_NOT_MET: PRECONDITION_NOT_MET
     *  - None
     * - other: PORT_ERROR
     *  - None
     *
     * @param status BufferStatus
     * @param data cdrMemoryStream
     * @return DataPortStatu ^̃^[R[h
     *
     * @else
     * @brief Convertion from BufferStatus to DataPortStatus
     * 
     * This function converts return value from the buffer to
     * DataPortStatus::Enum typed return value. The conversion rule is
     * as follows. Callback functions are also shown, if it exists.
     * 
     * - BUFFER_OK: PORT_OK
     *  - None
     * - BUFFER_ERROR: BUFFER_ERROR
     *  - None
     * - BUFFER_FULL: BUFFER_FULL
     *  - onBufferFull()
     * - NOT_SUPPORTED: PORT_ERROR
     *  - None
     * - TIMEOUT: BUFFER_TIMEOUT
     *  - onBufferWriteTimeout()
     * - PRECONDITION_NOT_MET: PRECONDITION_NOT_MET
     *  - None
     * - other: PORT_ERROR
     *  - None
     *
     * @param status BufferStatus
     * @param data cdrMemoryStream
     * @return DataPortStatus typed return code
     *
     * @endif
     */
    ReturnCode convertReturn(BufferStatus::Enum status,
                             const cdrMemoryStream& data);


    /*!
     * @if jp
     * @brief DataPortStatusɏ]ăXi֒ʒm֐ĂяoB
     *
     * @param status DataPortStatus
     * @param data cdrMemoryStream
     * @return ^[R[h
     *
     * @else
     * @brief Call listeners according to the DataPortStatus
     *
     * @param status DataPortStatus
     * @param data cdrMemoryStream
     * @return Return code
     *
     * @endif
     */
    ReturnCode invokeListener(DataPortStatus::Enum status,
                              const cdrMemoryStream& data);

    /*!
     * @if jp
     * @brief ON_BUFFER_WRITẼXi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_BUFFER_WRITE event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onBufferWrite(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_BUFFER_WRITE].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_BUFFER_FULLXiփCxgʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_BUFFER_FULL event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onBufferFull(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_BUFFER_FULL].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_BUFFER_WRITE_TIMEOUT̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_BUFFER_WRITE_TIMEOUT event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onBufferWriteTimeout(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_BUFFER_WRITE_TIMEOUT].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_BUFFER_READ̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_BUFFER_READ event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onBufferRead(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_BUFFER_READ].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_SEND̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_SEND event to listners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onSend(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_SEND].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_RECEIVED̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_RECEIVED event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onReceived(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_RECEIVED].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_RECEIVER_FULL̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_RECEIVER_FULL event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onReceiverFull(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_RECEIVER_FULL].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_RECEIVER_TIMEOUT̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_RECEIVER_TIMEOUT event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onReceiverTimeout(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_RECEIVER_TIMEOUT].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_RECEIVER_ERROR̃Xi֒ʒmB 
     * @param data cdrMemoryStream
     * @else
     * @brief Notify an ON_RECEIVER_ERROR event to listeners
     * @param data cdrMemoryStream
     * @endif
     */
    inline void onReceiverError(const cdrMemoryStream& data)
    {
      m_listeners->
        connectorData_[ON_RECEIVER_ERROR].notify(m_profile, data);
    }

    /*!
     * @if jp
     * @brief ON_BUFFER_EMPTỸXi֒ʒmB 
     * @else
     * @brief Notify an ON_BUFFER_EMPTY event to listeners
     * @endif
     */
    inline void onBufferEmpty()
    {
      m_listeners->
        connector_[ON_BUFFER_EMPTY].notify(m_profile);
    }

    /*!
     * @if jp
     * @brief ON_SENDER_EMPTỸXi֒ʒmB 
     * @else
     * @brief Notify an ON_SENDER_EMPTY event to listeners
     * @endif
     */
    inline void onSenderEmpty()
    {
      m_listeners->
        connector_[ON_SENDER_EMPTY].notify(m_profile);
    }

    /*!
     * @if jp
     * @brief ON_SENDER_ERROR̃Xi֒ʒmB 
     * @else
     * @brief Notify an ON_SENDER_ERROR event to listeners
     * @endif
     */
    inline void onSenderError()
    {
      m_listeners->
        connector_[ON_SENDER_ERROR].notify(m_profile);
    }


  private:
    bool bufferIsEmpty()
    {
      if (m_buffer->empty() && !m_readback)
        {
          RTC_DEBUG(("buffer empty"));
          onBufferEmpty();
          onSenderEmpty();
          return true;
        }
      return false;
    }

    Logger rtclog;
    InPortConsumer* m_consumer;
    CdrBufferBase* m_buffer;
    ConnectorInfo m_profile;
    coil::PeriodicTaskBase* m_task;
    ConnectorListeners* m_listeners;
    ReturnCode m_retcode;
    Mutex m_retmutex;
    Policy m_pushPolicy;
    int m_skipn;
    bool m_active;
    bool m_readback;
    int m_leftskip;
  };
};     // namespace RTC

extern "C"
{
  void DLL_EXPORT PublisherPeriodicInit();
};

#endif // RTC_PUBLISHERPERIODIC_H

